@@ -431,6 +431,12 @@ extern int flag_short_wchar;
 extern int flag_lvalue_cast_assign;
 /* APPLE LOCAL end lvalue cast */
 
+/* APPLE LOCAL begin 5612787 mainline sse4 */
+/* Nonzero means allow implicit conversions between vectors with
+   differing numbers of subparts and/or differing element types.  */
+extern int flag_lax_vector_conversions;
+/* APPLE LOCAL end 5612787 mainline sse4 */
+
 /* Nonzero means allow Microsoft extensions without warnings or errors.  */
 extern int flag_ms_extensions;
 
@@ -460,6 +466,12 @@ extern int warn_unknown_pragmas; /* Tri state variable.  */
 
 extern int warn_format;
 
+/* LLVM LOCAL begin */
+/* Warn about possible security problems with format functions  */
+
+extern int warn_format_security;
+/* LLVM LOCAL end */
+
 /* APPLE LOCAL begin disable_typechecking_for_spec_flag */
 /* This makes type conflicts a warning, instead of an error,
    to work around some problems with SPEC.  */
@@ -719,7 +731,8 @@ extern tree c_sizeof_or_alignof_type (tree, bool, int);
 extern tree c_alignof_expr (tree);
 /* Print an error message for invalid operands to arith operation CODE.
    NOP_EXPR is used as a special case (see truthvalue_conversion).  */
-extern void binary_op_error (enum tree_code);
+/* APPLE LOCAL 5612787 mainline sse4 */
+extern void binary_op_error (enum tree_code, tree, tree);
 extern tree fix_string_type (tree);
 struct varray_head_tag;
 extern void constant_expression_warning (tree);
@@ -774,8 +787,6 @@ extern HOST_WIDE_INT c_common_to_target_charset (HOST_WIDE_INT);
 /* APPLE LOCAL begin IMA built-in decl merging fix (radar 3645899) */
 extern bool builtin_function_disabled_p (const char *);
 /* APPLE LOCAL end */
-/* APPLE LOCAL define this sensibly in all languages */
-extern bool c_flag_no_builtin (void);
 
 /* This is the basic parsing function.  */
 extern void c_parse_file (void);
@@ -868,7 +879,8 @@ extern tree finish_label_address_expr (tree);
 extern tree lookup_label (tree);
 extern tree lookup_name (tree);
 
-extern int vector_types_convertible_p (tree t1, tree t2);
+/* APPLE LOCAL 5612787 mainline sse4 */
+extern int vector_types_convertible_p (tree t1, tree t2, bool emit_lax_note);
 
 extern rtx c_expand_expr (tree, rtx, enum machine_mode, int, rtx *);
 
@@ -972,14 +984,20 @@ extern tree objc_common_type (tree, tree);
 extern tree objc_non_volatilized_type (tree);
 /* APPLE LOCAL radar 4697411 */
 extern void objc_volatilize_component_ref (tree, tree);
-extern bool objc_compare_types (tree, tree, int, tree);
-/* APPLE LOCAL radar 4229905 */
-extern bool objc_have_common_type (tree, tree, int, tree);
+/* APPLE LOCAL radar 6231433 */
+extern bool objc_compare_types (tree, tree, int, tree, const char *);
+/* APPLE LOCAL radar 4229905 - radar 6231433 */
+extern bool objc_have_common_type (tree, tree, int, tree, const char *);
 /* APPLE LOCAL radar 4133425 */
 extern bool objc_diagnose_private_ivar (tree);
 /* APPLE LOCAL radar 4507230 */
 bool objc_type_valid_for_messaging (tree);
 extern void objc_volatilize_decl (tree);
+/* LLVM LOCAL begin - rdar 6551276 */
+#ifdef ENABLE_LLVM
+extern tree objc_build_volatilized_type (tree);
+#endif
+/* LLVM LOCAL end - rdar 6551276 */
 extern bool objc_type_quals_match (tree, tree);
 extern tree objc_rewrite_function_call (tree, tree);
 extern tree objc_message_selector (void);
@@ -1015,6 +1033,11 @@ extern void objc_start_class_implementation (tree, tree);
 extern void objc_start_category_implementation (tree, tree);
 /* APPLE LOCAL radar 4592503 */
 extern void objc_checkon_weak_attribute (tree);
+/* APPLE LOCAL begin radar 5847976 */
+extern tree build_block_object_assign_call_exp (tree, tree, int);
+extern tree build_block_object_dispose_call_exp (tree, int);
+extern int objc_is_gcable_type (tree);
+/* APPLE LOCAL end radar 5847976 */
 extern void objc_continue_implementation (void);
 extern void objc_finish_implementation (void);
 extern void objc_set_visibility (int);
@@ -1059,6 +1082,8 @@ void objc_add_property_variable (tree);
 tree objc_build_property_reference_expr (tree, tree);
 bool objc_property_reference_expr (tree);
 /* APPLE LOCAL end radar 5285911 */
+/* APPLE LOCAL radar 5802025 */
+tree objc_build_property_getter_func_call (tree);
 tree objc_build_setter_call (tree, tree);
 /* APPLE LOCAL end C* property (Radar 4436866) */
 /* APPLE LOCAL radar 4712269 */
@@ -1086,8 +1111,6 @@ tree objc_v2_bitfield_ivar_bitpos (tree);
 bool objc_check_format_nsstring (tree, unsigned HOST_WIDE_INT, bool *);
 /* APPLE LOCAL end 4985544 */
 
-/* APPLE LOCAL radar 2996215 */
-tree objc_create_init_utf16_var (const unsigned char *, size_t, size_t *);
 /* APPLE LOCAL radar 5202926 */
 bool objc_anonymous_local_objc_name (const char *);
 /* APPLE LOCAL begin radar 5195402 */
@@ -1113,6 +1136,112 @@ extern bool check_missing_format_attribute (tree, tree);
 extern tree vector_constructor_from_expr (tree, tree);
 /* APPLE LOCAL end AltiVec */
 
+/* APPLE LOCAL begin radar 5847976 */
+/* Runtime support functions used by compiler when generating copy/dispose helpers */
+enum {
+    BLOCK_FIELD_IS_OBJECT   =  3,  /* id, NSObject, __attribute__((NSObject)), block, ... */
+    BLOCK_FIELD_IS_BLOCK    =  7,  /* a block variable */
+    BLOCK_FIELD_IS_BYREF    =  8,  /* the on stack structure holding the __block variable */
+    BLOCK_FIELD_IS_WEAK     = 16,  /* declared __weak, only used in byref copy helpers */
+    BLOCK_BYREF_CALLER      = 128, /* called from __block (byref) copy/dispose support routines */
+    BLOCK_BYREF_CURRENT_MAX = 256
+};
+/* APPLE LOCAL end radar 5847976 */
+/* APPLE LOCAL begin radar 5732232 - blocks */
+enum {
+     BLOCK_NEEDS_FREE =        (1 << 24),
+     BLOCK_HAS_COPY_DISPOSE =  (1 << 25),
+     /* APPLE LOCAL radar 6214617 */
+     BLOCK_HAS_CXX_OBJ =       (1 << 26), 
+     BLOCK_IS_GC =             (1 << 27),
+     /* APPLE LOCAL radar 5822844 */
+     BLOCK_IS_GLOBAL = 	       (1 << 28),
+     /* APPLE LOCAL radar 5847213 */
+     BLOCK_HAS_DESCRIPTOR =    (1 << 29)
+};
+
+struct block_sema_info {
+  tree helper_func_decl;
+  tree copy_helper_func_decl;
+  tree destroy_helper_func_decl;
+  tree block_arg_ptr_type;
+  /* This is for C.  */
+  struct c_arg_info * arg_info;
+  tree block_ref_decl_list;
+  tree block_byref_decl_list;
+  /* APPLE LOCAL radar 5803600 */
+  tree block_byref_global_decl_list;
+  tree block_original_ref_decl_list;
+  /* APPLE LOCAL radar 5847213 - tree block_original_byref_decl_list is removed. */
+  tree block_body;
+  bool BlockHasCopyDispose;
+  /* APPLE LOCAL radar 6214617 */
+  bool BlockImportsCxxObjects;
+  /* APPLE LOCAL radar 6185344 */
+  bool block_has_return_type; /* When true, block has a declared return type. */
+
+  /* the_scope - This is the scope for the block itself, which
+     contains arguments etc.  Use only for C.  */
+  struct c_scope *the_scope;
+  /* Same as the above, only for C++.  */
+  struct cp_binding_level *cp_the_scope;
+
+  /* return_type - This will get set to block result type, by looking
+     at return types, if any, in the block body. */
+  tree return_type;
+
+  /* prev_block_info - If this is nested inside another block, this points
+     to the outer block. */
+  struct block_sema_info *prev_block_info;
+};
+
+extern struct block_sema_info *cur_block;
+extern tree build_helper_func_decl (tree, tree);
+extern tree build_block_byref_decl (tree, tree, tree);
+extern tree build_block_ref_decl (tree, tree);
+extern tree begin_block (void);
+extern struct block_sema_info *finish_block (tree);
+extern bool in_imm_block (void);
+extern bool lookup_name_in_block (tree, tree*);
+extern void push_to_top_level (void);
+extern void pop_from_top_level (void);
+extern void start_block_helper_function (tree func_decl);
+extern void block_build_prologue (struct block_sema_info *block_impl);
+extern tree c_finish_return (tree);
+extern bool block_requires_copying (tree);
+/* APPLE LOCAL begin radar 5803600 */
+extern void add_block_global_byref_list (tree);
+extern bool in_block_global_byref_list (tree);
+/* APPLE LOCAL end radar 5803600 */
+/* APPLE LOCAL end radar 5732232 - blocks */
+/* APPLE LOCAL begin radar 5932809 - copyable byref blocks */
+extern tree build_byref_local_var_access (tree, tree);
+extern tree do_digest_init (tree, tree);
+/* APPLE LOCAL end radar 5932809 - copyable byref blocks */
+/* APPLE LOCAL begin radar 6237713 */
+extern bool any_recognized_block_attribute (tree);
+/* APPLE LOCAL end radar 6237713 */
+
+/* APPLE LOCAL begin radar 5847213 */
+extern tree build_block_descriptor_type (bool);
+/* APPLE LOCAL end radar 5847213 */
+/* APPLE LOCAL begin radar 6083129 - byref escapes */
+extern tree build_block_byref_release_exp (tree);
+/* APPLE LOCAL end radar 6083129 - byref escapes */
+
+/* APPLE LOCAL radar 6040305 - blocks */
+extern tree build_indirect_object_id_exp (tree);
+/* APPLE LOCAL begin radar 6212722 */
+extern tree array_to_pointer_conversion (tree);
+extern tree function_to_pointer_conversion (tree);
+/* APPLE LOCAL end radar 6212722 */
+
+/* APPLE LOCAL radar 6160536 */
+extern tree build_block_helper_name (int);
+
+/* APPLE LOCAL radar 6353006  */
+extern tree c_build_generic_block_struct_type (void);
+
 /* In c-omp.c  */
 extern tree c_finish_omp_master (tree);
 extern tree c_finish_omp_critical (tree, tree);
