@@ -54,7 +54,8 @@ typedef struct align_stack GTY(())
 static GTY(()) struct align_stack * alignment_stack;
 
 /* LLVM LOCAL begin */
-#ifndef TARGET_OVERRIDE_PRAGMA_PACK_HANDLER
+/* APPLE LOCAL Macintosh alignment (disable) */
+#if defined(ENABLE_LLVM) && !defined(TARGET_OVERRIDE_PRAGMA_PACK_HANDLER)
 /* LLVM LOCAL end */
 
 #ifdef HANDLE_PRAGMA_PACK
@@ -248,9 +249,8 @@ handle_pragma_pack (cpp_reader * ARG_UNUSED (dummy))
     }
 }
 #endif  /* HANDLE_PRAGMA_PACK */
-
 /* LLVM LOCAL begin */
-#endif /* TARGET_OVERRIDE_PRAGMA_PACK_HANDLER */
+#endif /* ENABLE_LLVM && !TARGET_OVERRIDE_PRAGMA_PACK_HANDLER */
 /* LLVM LOCAL end */
 
 static GTY(()) tree pending_weaks;
@@ -821,7 +821,11 @@ init_pragma (void)
 				PRAGMA_GCC_PCH_PREPROCESS, false, false);
 
 /* LLVM LOCAL begin */
-#ifndef TARGET_OVERRIDE_PRAGMA_PACK_HANDLER
+/* APPLE LOCAL begin Macintosh alignment (disable section) 2002-1-22 --ff */
+  /* Remove the handling of pragma pack here because it is handled
+     in config/darwin-c.c.  */
+#if defined(ENABLE_LLVM) && !defined(TARGET_OVERRIDE_PRAGMA_PACK_HANDLER)
+/* APPLE LOCAL end Macintosh alignment (disable section) 2002-1-22 --ff */
 /* LLVM LOCAL end */
 #ifdef HANDLE_PRAGMA_PACK
 #ifdef HANDLE_PRAGMA_PACK_WITH_EXPANSION
@@ -830,9 +834,8 @@ init_pragma (void)
   c_register_pragma (0, "pack", handle_pragma_pack);
 #endif
 #endif
-/* LLVM LOCAL begin */
-#endif
-/* LLVM LOCAL end */
+/* LLVM LOCAL */
+#endif /* ENABLE_LLVM && !TARGET_OVERRIDE_PRAGMA_PACK_HANDLER */
 #ifdef HANDLE_PRAGMA_WEAK
   c_register_pragma (0, "weak", handle_pragma_weak);
 #endif
