@@ -166,8 +166,11 @@ Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 #include "tree-gimple.h"
 #include "tree-pass.h"
 #include "output.h"
-/* LLVM LOCAL */
+/* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
 #include "llvm.h"
+#endif
+/* LLVM LOCAL end */
 
 static void cgraph_expand_all_functions (void);
 static void cgraph_mark_functions_to_output (void);
@@ -232,9 +235,19 @@ decide_is_function_needed (struct cgraph_node *node, tree decl)
      in the original implementation and it is unclear whether we want
      to change the behavior here.  */
   if (((TREE_PUBLIC (decl)
+/* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
+    /* Don't keep static functions at -O0.  Note that gcc-4.0 removes such
+       functions; gcc-4.2 does not.  Radar 6037815.  See also PR 24561.
+
+       For this to be effective, -funit-at-a-time must also be on.  This
+       is the default in C++/ObjC++, but not C/ObjC.  */
 	|| (!optimize && !node->local.disregard_inline_limits
 	    && !DECL_DECLARED_INLINE_P (decl)
-	    && !node->origin))
+	    && !node->origin)
+#endif
+        )
+/* LLVM LOCAL end */
       && !flag_whole_program)
       && !DECL_COMDAT (decl) && !DECL_EXTERNAL (decl))
     return true;
@@ -371,7 +384,8 @@ cgraph_assemble_pending_functions (void)
       n->next_needed = NULL;
       if (!n->global.inlined_to
 	  && !n->alias
-	  && !DECL_EXTERNAL (n->decl))
+          /* LLVM LOCAL extern inline */
+	  && !IS_EXTERN_INLINE (n->decl))
 	{
 	  cgraph_expand_function (n);
 	  output = true;
@@ -460,6 +474,19 @@ cgraph_lower_function (struct cgraph_node *node)
   node->lowered = true;
 }
 
+/* APPLE LOCAL begin radar 6305545 */
+/** lower_if_nested_functions - This routine is called from cplus side only.
+    Its purpose is to lower block helper (or any other nested function)
+    which may have been nested in a constructor or destructor. We have to
+    do this because structors are cloned and are not lowered themselves (which
+    is the only way to lower the nested functions). */
+void 
+lower_if_nested_functions (tree decl)
+{
+    lower_nested_functions (decl, true);
+}
+/* APPLE LOCAL end radar 6305545 */
+
 /* DECL has been parsed.  Take it, queue it, compile it at the whim of the
    logic in effect.  If NESTED is true, then our caller cannot stand to have
    the garbage collector run at the moment.  We would need to either create
@@ -484,7 +511,8 @@ cgraph_finalize_function (tree decl, bool nested)
   node->local.finalized = true;
   node->lowered = DECL_STRUCT_FUNCTION (decl)->cfg != NULL;
   if (node->nested)
-    lower_nested_functions (decl);
+    /* APPLE LOCAL radar 6305545 */
+    lower_nested_functions (decl, false);
   gcc_assert (!node->nested);
 
   /* If not unit at a time, then we need to create the call graph
@@ -820,7 +848,8 @@ verify_cgraph_node (struct cgraph_node *node)
 
   if (node->analyzed
       && DECL_SAVED_TREE (node->decl) && !TREE_ASM_WRITTEN (node->decl)
-      && (!DECL_EXTERNAL (node->decl) || node->global.inlined_to))
+      /* LLVM LOCAL extern inline */ 
+      && (!IS_EXTERN_INLINE (node->decl) || node->global.inlined_to))
     {
       if (this_cfun->cfg)
 	{
@@ -1265,22 +1294,24 @@ cgraph_mark_functions_to_output (void)
 	  && (node->needed
 	      || (e && node->reachable))
 	  && !TREE_ASM_WRITTEN (decl)
-	  && !DECL_EXTERNAL (decl))
+          /* LLVM LOCAL extern inline */
+	  && !IS_EXTERN_INLINE (decl))
 	node->output = 1;
       else
 	{
 	  /* We should've reclaimed all functions that are not needed.  */
 #ifdef ENABLE_CHECKING
 	  if (!node->global.inlined_to && DECL_SAVED_TREE (decl)
-	      && !DECL_EXTERNAL (decl))
+              /* LLVM LOCAL extern inline */
+	      && !IS_EXTERN_INLINE (decl))
 	    {
 	      dump_cgraph_node (stderr, node);
 	      internal_error ("failed to reclaim unneeded function");
 	    }
 #endif
+          /* LLVM LOCAL extern inline */
 	  gcc_assert (node->global.inlined_to || !DECL_SAVED_TREE (decl)
-		      || DECL_EXTERNAL (decl));
-
+		      || IS_EXTERN_INLINE (decl));
 	}
 
     }
@@ -1568,6 +1599,8 @@ cgraph_preserve_function_body_p (tree decl)
   return false;
 }
 
+/* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
 static void
 ipa_passes (void)
 {
@@ -1577,6 +1610,8 @@ ipa_passes (void)
   execute_ipa_pass_list (all_ipa_passes);
   bitmap_obstack_release (NULL);
 }
+#endif
+/* LLVM LOCAL end */
 
 /* Perform simple optimizations based on callgraph.  */
 
@@ -1614,9 +1649,13 @@ cgraph_optimize (void)
       dump_cgraph (cgraph_dump_file);
     }
     
+  /* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
   /* Don't run the IPA passes if there was any error or sorry messages.  */
   if (errorcount == 0 && sorrycount == 0)
     ipa_passes ();
+#endif
+  /* LLVM LOCAL end */
 
   /* This pass remove bodies of extern inline functions we never inlined.
      Do this later so other IPA passes see what is really going on.  */
