@@ -338,10 +338,12 @@ darwin_pragma_unused (cpp_reader *pfile ATTRIBUTE_UNUSED)
   tree decl, x;
   int tok;
 
-  if (pragma_lex (&x) != CPP_OPEN_PAREN)
+  /* APPLE LOCAL 5979888 */
+  if ((tok=pragma_lex (&x)) != CPP_OPEN_PAREN)
     BAD ("missing '(' after '#pragma unused', ignoring");
 
-  while (1)
+  /* APPLE LOCAL 5979888 */
+  while (tok != CPP_EOF && tok != CPP_CLOSE_PAREN)
     {
       tok = pragma_lex (&decl);
       if (tok == CPP_NAME && decl)
@@ -410,7 +412,8 @@ darwin_pragma_reverse_bitfields (cpp_reader *pfile ATTRIBUTE_UNUSED)
 /* APPLE LOCAL end pragma reverse_bitfields */
 
 /* APPLE LOCAL begin optimization pragmas 3124235/3420242 */
-varray_type va_opt;
+/* LLVM LOCAL put va_opt in GC memory 7210265 */
+static GTY(()) varray_type va_opt;
 
 static void
 push_opt_level (int level, int size)
@@ -930,7 +933,8 @@ find_subframework_header (cpp_reader *pfile, const char *header, cpp_dir **dirp)
    so '10.4.2' becomes 1042.
    Print a warning if the version number is not known.  */
 static const char *
-version_as_macro (void)
+/* APPLE LOCAL ARM 5683689 */
+macosx_version_as_macro (void)
 {
   static char result[] = "1000";
 
@@ -960,6 +964,81 @@ version_as_macro (void)
   return "1000";
 }
 
+/* APPLE LOCAL begin ARM 5683689 */
+/* Return the value of darwin_iphoneos_version_min suitable for the
+   __ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__ macro.  Unlike the
+   __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ macros, minor version
+   numbers are left-zero-padded.  e.g., '1.2.3' becomes 10203.
+   The last/third version number (patch level?) is optional, and
+   defaults to '00' if not specified.  In the case of a parse error,
+   print a warning and return 10200.  */
+static const char *
+iphoneos_version_as_macro (void)
+{
+  static char result[sizeof ("99.99.99") + 1];
+  const char *src_ptr = darwin_iphoneos_version_min;
+  char *result_ptr = &result[0];
+
+  if (! darwin_iphoneos_version_min)
+    goto fail;
+
+  if (! ISDIGIT (*src_ptr))
+    goto fail;
+
+  /* Copy over the major version number.  */
+  *result_ptr++ = *src_ptr++;
+
+  if (ISDIGIT (*src_ptr))
+    *result_ptr++ = *src_ptr++;
+
+  if (*src_ptr != '.')
+    goto fail;
+
+  src_ptr++;
+
+  /* Start parsing the minor version number.  */
+  if (! ISDIGIT (*src_ptr))
+    goto fail;
+
+  /* Zero-pad a single-digit value, or copy a two-digit value.  */
+  *result_ptr++ = ISDIGIT (*(src_ptr + 1)) ? *src_ptr++ : '0';
+  *result_ptr++ = *src_ptr++;
+
+  /* Parse the third version number (patch level?)  */
+  if (*src_ptr == '\0')
+    {
+      /* Not present -- default to zeroes.  */
+      *result_ptr++ = '0';
+      *result_ptr++ = '0';
+    }
+  else if (*src_ptr == '.')
+    {
+      src_ptr++;
+
+      if (! ISDIGIT (*src_ptr))
+	goto fail;
+
+      /* Zero-pad a single-digit value, or copy a two-digit value.  */
+      *result_ptr++ = ISDIGIT (*(src_ptr + 1)) ? *src_ptr++ : '0';
+      *result_ptr++ = *src_ptr++;
+    }
+  else
+    goto fail;
+
+  /* Verify and copy the terminating NULL.  */
+  if (*src_ptr != '\0')
+    goto fail;
+ 
+  *result_ptr++ = '\0'; 
+  return result;
+  
+ fail:
+  error ("Unknown value %qs of -miphoneos-version-min",
+	 darwin_iphoneos_version_min);
+  return "10200";
+}
+/* APPLE LOCAL end ARM 5683689 */
+
 /* Define additional CPP flags for Darwin.   */
 
 #define builtin_define(TXT) cpp_define (pfile, TXT)
@@ -973,10 +1052,14 @@ darwin_cpp_builtins (cpp_reader *pfile)
   /* APPLE LOCAL Apple version */
   /* Don't define __APPLE_CC__ here.  */
 
-  /* APPLE LOCAL begin mainline 2007-02-20 5005743 */
-  builtin_define_with_value ("__ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__",
-			     version_as_macro(), false);
-  /* APPLE LOCAL end mainline 2007-02-20 5005743 */
+  /* APPLE LOCAL begin ARM 5683689 */
+  if (darwin_macosx_version_min)
+    builtin_define_with_value ("__ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__",
+			       macosx_version_as_macro(), false);
+  else
+    builtin_define_with_value ("__ENVIRONMENT_IPHONE_OS_VERSION_MIN_REQUIRED__",
+			       iphoneos_version_as_macro(), false);
+  /* APPLE LOCAL end ARM 5683689 */
 
   /* APPLE LOCAL begin constant cfstrings */
   if (darwin_constant_cfstrings)
@@ -989,7 +1072,8 @@ darwin_cpp_builtins (cpp_reader *pfile)
     }
   /* APPLE LOCAL end pascal strings */
   /* APPLE LOCAL begin ObjC GC */
-  if (flag_objc_gc)
+  /* APPLE LOCAL radar 5914395 */
+  if (flag_objc_gc || flag_objc_gc_only)
     {
       builtin_define ("__strong=__attribute__((objc_gc(strong)))");
       builtin_define ("__weak=__attribute__((objc_gc(weak)))");
@@ -998,9 +1082,18 @@ darwin_cpp_builtins (cpp_reader *pfile)
   else
     {
       builtin_define ("__strong=");
-      builtin_define ("__weak=");
+      /* APPLE LOCAL radar 5847976 */
+      builtin_define ("__weak=__attribute__((objc_gc(weak)))");
     }
   /* APPLE LOCAL end ObjC GC */
+  /* APPLE LOCAL begin radar 5932809 - copyable byref blocks */
+  if (flag_blocks) {
+    builtin_define ("__block=__attribute__((__blocks__(byref)))");
+  }
+  /* APPLE LOCAL radar 6230656 */
+  /* code removed */
+  /* APPLE LOCAL end radar 5932809 - copyable byref blocks */
+
   /* APPLE LOCAL begin C* warnings to easy porting to new abi */
   if (flag_objc_abi == 2)
     builtin_define ("__OBJC2__");
@@ -1011,9 +1104,9 @@ darwin_cpp_builtins (cpp_reader *pfile)
   /* APPLE LOCAL radar 4899595 */
   builtin_define ("OBJC_NEW_PROPERTIES");
   /* APPLE LOCAL end radar 5072864 */
+/* APPLE LOCAL begin confused diff */
 }
-/* APPLE LOCAL confused diff */
-
+/* APPLE LOCAL end confused diff */
 /* APPLE LOCAL begin iframework for 4.3 4094959 */
 bool
 darwin_handle_c_option (size_t code, const char *arg, int value ATTRIBUTE_UNUSED)
@@ -1055,6 +1148,13 @@ objc_check_format_cfstring (tree argument,
                             bool *no_add_attrs)
 {
   unsigned HOST_WIDE_INT i;
+  /* APPLE LOCAL begin 6212507 */
+  if (format_num < 1)
+    {
+      error ("argument number of CFString format cannot be less than one");
+      return false;
+    }
+  /* APPLE LOCAL end 6212507 */
   for (i = 1; i != format_num; i++)
     {
       if (argument == 0)
@@ -1062,6 +1162,14 @@ objc_check_format_cfstring (tree argument,
        argument = TREE_CHAIN (argument);
     }
 
+  /* LLVM LOCAL begin 7020016 */
+  if (argument == NULL_TREE)
+    {
+      error ("argument number of CFString format too large");
+      *no_add_attrs = true;
+      return false;
+    }
+  /* LLVM LOCAL end 7020016 */
   if (!objc_check_cfstringref_type (TREE_VALUE (argument)))
     {
       error ("format CFString argument not an 'CFStringRef' type");
@@ -1071,3 +1179,87 @@ objc_check_format_cfstring (tree argument,
   return true;
 }
 /* APPLE LOCAL end radar 4985544 - radar 5096648 - radar 5195402 */
+
+/* APPLE LOCAL begin radar 2996215 - 6068877 */
+/* wrapper to call libcpp's conversion routine. */
+bool
+cvt_utf8_utf16 (const unsigned char *inbuf, size_t length, 
+		     unsigned char **uniCharBuf, size_t *numUniChars)
+{
+  return cpp_utf8_utf16 (parse_in, inbuf, length, uniCharBuf, numUniChars);
+}
+/* This routine declares static char __utf16_string [numUniChars] in __TEXT,__ustring
+   section and initializes it with uniCharBuf[numUniChars] characters.
+*/ 
+tree
+create_init_utf16_var (const unsigned char *inbuf, size_t length, size_t *numUniChars)
+{
+  size_t l;
+  tree decl, type, init;
+  tree initlist = NULL_TREE;
+  tree attribute; 
+  const char *section_name = "__TEXT,__ustring";
+  int len = strlen (section_name);
+  unsigned char *uniCharBuf;
+  static int num;
+  const char *name_prefix = "__utf16_string_";
+  char *name;
+  int embedNull = 0;
+
+  if (!cvt_utf8_utf16 (inbuf, length, &uniCharBuf, numUniChars))
+    return NULL_TREE;
+
+  /* LLVM LOCAL begin 7589850. */
+  /* ustring with embedded null should go into __const. It should not be forced
+     into "__TEXT,__ustring" section. */
+  for (l = 0; l < length; l++) {
+    if (!inbuf[l]) {
+      embedNull = 1;
+      break;
+    }
+  }
+  /* LLVM LOCAL end 7589850. */
+
+  for (l = 0; l < *numUniChars; l++)
+    initlist = tree_cons (NULL_TREE, build_int_cst (char_type_node, uniCharBuf[l]), initlist);
+  /* LLVM LOCAL utf16 has two trailing nulls 7095855 */
+  type = build_array_type (char_type_node,
+                           build_index_type (build_int_cst (NULL_TREE, *numUniChars + 1)));
+  name = (char *)alloca (strlen (name_prefix) + 10);
+  sprintf (name, "%s%d", name_prefix, ++num);
+  decl = build_decl (VAR_DECL, get_identifier (name), type);
+  TREE_STATIC (decl) = 1;
+  DECL_INITIAL (decl) = error_mark_node;  /* A real initializer is coming... */
+  DECL_IGNORED_P (decl) = 1;
+  DECL_ARTIFICIAL (decl) = 1;
+  DECL_CONTEXT (decl) = NULL_TREE;
+  /* LLVM LOCAL begin 7115749 this object is constant. */
+  TREE_CONSTANT (decl) = 1;
+  TREE_READONLY (decl) = 1;
+  /* LLVM LOCAL end */
+
+  /* LLVM LOCAL 7589850 */
+  if (!embedNull) {
+    attribute = tree_cons (NULL_TREE, build_string (len, section_name), NULL_TREE);
+    attribute = tree_cons (get_identifier ("section"), attribute, NULL_TREE);
+    decl_attributes (&decl, attribute, 0);
+  }
+  attribute = tree_cons (NULL_TREE, build_int_cst (NULL_TREE, 2), NULL_TREE);
+  attribute = tree_cons (get_identifier ("aligned"), attribute, NULL_TREE);
+  decl_attributes (&decl, attribute, 0);
+  init = build_constructor_from_list (type, nreverse (initlist));
+  TREE_CONSTANT (init) = 1;
+  TREE_STATIC (init) = 1;
+  TREE_READONLY (init) = 1;
+  if (c_dialect_cxx ())
+    TREE_TYPE (init) = NULL_TREE;
+  finish_decl (decl, init, NULL_TREE);
+  /* Ensure that the variable actually gets output.  */
+  mark_decl_referenced (decl);
+  /* Mark the decl to avoid "defined but not used" warning.  */
+  TREE_USED (decl) = 1;
+  return decl;
+}
+/* APPLE LOCAL end radar 2996215 - 6068877 */
+/* LLVM LOCAL put va_opt in GC memory 7210265 */
+#include "gt-darwin-c.h"
