@@ -44,6 +44,10 @@ Boston, MA 02110-1301, USA.  */
 extern int machopic_symbol_defined_p (rtx);
 /* APPLE LOCAL end dynamic-no-pic */
 
+/* APPLE LOCAL begin axe stubs 5571540 */
+extern int darwin_stubs;
+/* APPLE LOCAL end axe stubs 5571540 */
+
 /* APPLE LOCAL begin mainline 2006-11-01 3334812 */
 /* Don't assume anything about the header files.  */
 #define NO_IMPLICIT_EXTERN_C
@@ -120,8 +124,18 @@ extern GTY(()) int darwin_ms_struct;
    command-line option translations specific to the particular target
    architecture.  */
 
+/* LLVM LOCAL - begin */
+#ifdef ENABLE_LLVM
+#define LLVM_FLAG  { "-mllvm", "-Zmllvm" },
+#else
+#define LLVM_FLAG
+#endif
+/* LLVM LOCAL - end */
+
 #define TARGET_OPTION_TRANSLATE_TABLE \
   { "-all_load", "-Zall_load" },  \
+  /* APPLE LOCAL 7519550 -force_load */ \
+  { "-force_load", "-Zforce_load" },  \
   { "-allowable_client", "-Zallowable_client" },  \
   { "-arch_errors_fatal", "-Zarch_errors_fatal" },  \
   { "-bind_at_load", "-Zbind_at_load" },  \
@@ -144,7 +158,8 @@ extern GTY(()) int darwin_ms_struct;
   { "-seg_addr_table_filename", "-Zfn_seg_addr_table_filename" }, \
   /* APPLE LOCAL mainline */ \
   { "-umbrella", "-Zumbrella" }, \
-  { "-fapple-kext", "-fapple-kext -static -Wa,-static" }, \
+  /* APPLE LOCAL kext weak_import 5935650 */ \
+  { "-fapple-kext", "-fapple-kext -static" }, \
   { "-filelist", "-Xlinker -filelist -Xlinker" },  \
   { "-findirect-virtual-calls", "-fapple-kext" }, \
   { "-flat_namespace", "-Zflat_namespace" },  \
@@ -156,18 +171,21 @@ extern GTY(()) int darwin_ms_struct;
   { "-init", "-Zinit" },  \
   { "-install_name", "-Zinstall_name" },  \
   /* LLVM LOCAL */ \
-  { "-mllvm", "-Zmllvm" },  \
-  { "-mkernel", "-mkernel -static -Wa,-static" }, \
+  LLVM_FLAG  \
+  /* APPLE LOCAL kext weak_import 5935650 */ \
+  { "-mkernel", "-mkernel -static" }, \
   { "-multiply_defined_unused", "-Zmultiplydefinedunused" },  \
   { "-multiply_defined", "-Zmultiply_defined" },  \
   { "-multi_module", "-Zmulti_module" },  \
-  { "-static", "-static -Wa,-static" },  \
+  /* APPLE LOCAL begin kext weak_import 5935650 */ \
+  /* Removed -static */ \
+  /* APPLE LOCAL end kext weak_import 5935650 */ \
   /* APPLE LOCAL mainline */ \
   { "-shared", "-Zdynamiclib" }, \
   { "-single_module", "-Zsingle_module" },  \
   { "-unexported_symbols_list", "-Zunexported_symbols_list" }, \
-  /* APPLE LOCAL ObjC GC */ \
-  { "-fobjc-gc", "-fobjc-gc -Wno-non-lvalue-assign" }, \
+  /* APPLE LOCAL radar 6269491 */ \
+  /* code removed. */ \
   /* APPLE LOCAL begin constant cfstrings */	\
   { "-fconstant-cfstrings", "-mconstant-cfstrings" }, \
   { "-fno-constant-cfstrings", "-mno-constant-cfstrings" }, \
@@ -187,6 +205,13 @@ extern int darwin_running_cxx;
 extern GTY(()) int darwin_reverse_bitfields;
 /* APPLE LOCAL end pragma reverse_bitfields */
 
+/* APPLE LOCAL begin ARM 5683689 */
+enum darwin_version_type {
+  DARWIN_VERSION_MACOSX,
+  DARWIN_VERSION_IPHONEOS
+};
+/* APPLE LOCAL end ARM 5683689 */
+
 /* APPLE LOCAL AT&T-style stub 4164563 */
 #define MACHOPIC_ATT_STUB (darwin_macho_att_stub)
 
@@ -198,6 +223,11 @@ extern GTY(()) int darwin_reverse_bitfields;
 	warn_pointer_sign = 1;						\
 	CPP_OPTION (parse_in, pascal_strings) = 1;			\
       }									\
+    /* APPLE LOCAL begin ARM 5683689 */					\
+    if (darwin_macosx_version_min && darwin_iphoneos_version_min)	\
+      error ("-mmacosx-version-min not allowed with"			\
+             " -miphoneos-version-min");				\
+    /* APPLE LOCAL end ARM 5683689 */					\
     /* The c_dialect...() macros are not available to us here.  */	\
     darwin_running_cxx = (strstr (lang_hooks.name, "C++") != 0);	\
     /* APPLE LOCAL end constant cfstrings */				\
@@ -218,9 +248,19 @@ do {					\
 
 /* These compiler options take n arguments.  */
 
+/* LLVM LOCAL - begin */
+#ifdef ENABLE_LLVM
+#define LLVM_STRCMP(STR) !strcmp (STR, "Zmllvm") ? 1 :
+#else
+#define LLVM_STRCMP(STR)
+#endif
+/* LLVM LOCAL - end */
+ 
 #undef  WORD_SWITCH_TAKES_ARG
 #define WORD_SWITCH_TAKES_ARG(STR)              \
   (DEFAULT_WORD_SWITCH_TAKES_ARG (STR) ? 1 :    \
+   /* APPLE LOCAL 7519550 -force_load */ 	\
+   !strcmp (STR, "Zforce_load") ? 1 :     	\
    !strcmp (STR, "Zallowable_client") ? 1 :     \
    !strcmp (STR, "arch") ? 1 :                  \
    !strcmp (STR, "arch_only") ? 1 :             \
@@ -233,8 +273,8 @@ do {					\
    !strcmp (STR, "Zimage_base") ? 1 :           \
    !strcmp (STR, "Zinit") ? 1 :                 \
    !strcmp (STR, "Zinstall_name") ? 1 :         \
-  /* LLVM LOCAL */ \
-   !strcmp (STR, "Zmllvm") ? 1 :                \
+   /* LLVM LOCAL */                             \
+   LLVM_STRCMP(STR)                             \
    !strcmp (STR, "Zmultiplydefinedunused") ? 1 : \
    !strcmp (STR, "Zmultiply_defined") ? 1 :     \
    !strcmp (STR, "precomp-trustfile") ? 1 :     \
@@ -274,6 +314,12 @@ do {					\
 	flag_weak = 0;							\
 	/* No RTTI in kexts.  */					\
 	flag_rtti = 0;							\
+	/* APPLE LOCAL begin 5731065 */					\
+	if (flag_mkernel)						\
+	  flag_no_builtin = 1;						\
+	/* APPLE LOCAL end 5731065 */					\
+	/* APPLE LOCAL xmmintrin.h for kernel 4123064 */					\
+	flag_hosted = 0;						\
       }									\
   } while (0)
 
@@ -301,7 +347,8 @@ do {					\
 #define LINK_COMMAND_SPEC "\
 %{!fdump=*:%{!fsyntax-only:%{!precomp:%{!c:%{!M:%{!MM:%{!E:%{!S:\
     %(linker) %l %X %{d} %{s} %{t} %{Z} %{u*} \
-    %{A} %{e*} %{m} %{r} %{x} \
+"/* LLVM LOCAL lto */"\
+    %{A} %<emit-llvm %{e*} %{m} %{r} %{x} \
     %{o*}%{!o:-o a.out} \
     %{!A:%{!nostdlib:%{!nostartfiles:%S}}} \
     %{L*} %{fopenmp:%:include(libgomp.spec)%(link_gomp)}   \
@@ -309,15 +356,14 @@ do {					\
     %(link_libgcc) %o %{fprofile-arcs|fprofile-generate|fcreate-profile|coverage:-lgcov} \
 "/* APPLE LOCAL nested functions 4357979  */"\
     %{fnested-functions: -allow_stack_execute} \
-    %{!nostdlib:%{!nodefaultlibs:%(link_ssp) %G %L}} \
+"/* APPLE LOCAL prefer -lSystem 6645902 */"\
+    %{!nostdlib:%{!nodefaultlibs:%(link_ssp) %(link_gcc_c_sequence)}} \
 "/* APPLE LOCAL begin mainline 4.3 2006-10-31 4370146 */"\
     %{!A:%{!nostdlib:%{!nostartfiles:%E}}} %{T*} %{F*} }}}}}}}}\n\
 %{!fdump=*:%{!fsyntax-only:%{!c:%{!M:%{!MM:%{!E:%{!S:\
 "/* APPLE LOCAL end mainline 4.3 2006-10-31 4370146 */"\
-"/* APPLE LOCAL ARM 5342595 */"\
-"/* LLVM LOCAL do not use dsymutil with -O1 or higher */"\
     %{.c|.cc|.C|.cpp|.cp|.c++|.cxx|.CPP|.m|.mm: \
-    %{!O: %{!O1: %{!O2: %{!O3: %{!O4: %{!Os: %(darwin_dsymutil) }}}}}}}}}}}}}}"
+    %{g*:%{!gstabs*:%{!g0: dsymutil %{o*:%*}%{!o:a.out}}}}}}}}}}}}"
 /* APPLE LOCAL end mainline */
 
 #ifdef TARGET_SYSTEM_ROOT
@@ -334,6 +380,14 @@ do {					\
 /* Note that options taking arguments may appear multiple times on a
    command line with different arguments each time, so put a * after
    their names so all of them get passed.  */
+/* LLVM LOCAL - begin */
+#ifdef ENABLE_LLVM
+#define LLVM_LINK_SPEC "%{Zmllvm*:-mllvm %*}"
+#else
+#define LLVM_LINK_SPEC ""
+#endif
+/* LLVM LOCAL - end */
+
 /* APPLE LOCAL begin mainline */
 #define LINK_SPEC  \
   "%{static}%{!static:-dynamic} \
@@ -344,8 +398,7 @@ do {					\
      %{Zbundle:-bundle} \
      %{Zbundle_loader*:-bundle_loader %*} \
      %{client_name*} \
-     %{compatibility_version*:%e-compatibility_version only allowed with -dynamiclib\
-} \
+     %{compatibility_version*:%e-compatibility_version only allowed with -dynamiclib} \
      %{current_version*:%e-current_version only allowed with -dynamiclib} \
      %{Zforce_flat_namespace:-force_flat_namespace} \
      %{Zinstall_name*:%e-install_name only allowed with -dynamiclib} \
@@ -366,6 +419,8 @@ do {					\
      %{private_bundle:%e-private_bundle not allowed with -dynamiclib} \
     } \
    %{Zall_load:-all_load} \
+   "/* APPLE LOCAL 7519550 -force_load */" \
+   %{Zforce_load*:-force_load %*} \
    %{Zallowable_client*:-allowable_client %*} \
    %{Zbind_at_load:-bind_at_load} \
    %{Zarch_errors_fatal:-arch_errors_fatal} \
@@ -378,19 +433,27 @@ do {					\
    %{headerpad_max_install_names*} \
    %{Zimage_base*:-image_base %*} \
    %{Zinit*:-init %*} \
-   "/* APPLE LOCAL begin mainline 2007-02-20 5005743 */" \
-   %{!mmacosx-version-min=*:-macosx_version_min %(darwin_minversion)} \
+   "/* APPLE LOCAL begin ARM 5683689 */"\
+   %{!mmacosx-version-min=*: %{!miphoneos-version-min=*: %(darwin_ld_minversion)}} \
    %{mmacosx-version-min=*:-macosx_version_min %*} \
-   "/* APPLE LOCAL end mainline 2007-02-20 5005743 */" \
+   %{miphoneos-version-min=*:-iphoneos_version_min %*} \
+   "/* APPLE LOCAL end ARM 5683689 */"\
+   "/* LLVM LOCAL begin */\
+  LLVM_LINK_SPEC \
+   /* LLVM LOCAL end */" \
    %{nomultidefs} \
    %{Zmulti_module:-multi_module} %{Zsingle_module:-single_module} \
    %{Zmultiply_defined*:-multiply_defined %*} \
-   %{!Zmultiply_defined*:%{shared-libgcc: \
-     %:version-compare(< 10.5 mmacosx-version-min= -multiply_defined) \
-     %:version-compare(< 10.5 mmacosx-version-min= suppress)}} \
+   "/* APPLE LOCAL begin deletion 5023884 */" \
+   "/* APPLE LOCAL end deletion 5023884 */" \
    %{Zmultiplydefinedunused*:-multiply_defined_unused %*} \
    "/* APPLE LOCAL mainline 2007-06-01 5238485 */" \
    %{fpie:-pie} \
+   "/* LLVM LOCAL begin <rdar://problem/7651045> */" \
+   %{fPIE:-pie} \
+   %{fno-pie:-no_pie} \
+   %{fno-PIE:-no_pie} \
+   "/* LLVM LOCAL end <rdar://problem/7651045> */" \
    %{prebind} %{noprebind} %{nofixprebinding} %{prebind_all_twolevel_modules} \
    %{read_only_relocs} \
    %{sectcreate*} %{sectorder*} %{seg1addr*} %{segprot*} \
@@ -437,12 +500,16 @@ do {					\
 #define REAL_LIBGCC_SPEC						   \
 /* APPLE LOCAL libgcc_static.a  */					   \
    "%{static:-lgcc_static; static-libgcc: -lgcc_eh -lgcc;		   \
+      "/* APPLE LOCAL ARM 5683689 5681645 */"				   \
+      miphoneos-version-min=*: %(darwin_iphoneos_libgcc);		   \
       shared-libgcc|fexceptions|fgnu-runtime:				   \
        %:version-compare(!> 10.5 mmacosx-version-min= -lgcc_s.10.4)	   \
-       %:version-compare(>= 10.5 mmacosx-version-min= -lgcc_s.10.5)	   \
+       "/* APPLE LOCAL link optimizations 6499452 */"			   \
+       %:version-compare(>< 10.5 10.6 mmacosx-version-min= -lgcc_s.10.5)   \
        -lgcc;								   \
       :%:version-compare(>< 10.3.9 10.5 mmacosx-version-min= -lgcc_s.10.4) \
-       %:version-compare(>= 10.5 mmacosx-version-min= -lgcc_s.10.5)	   \
+       "/* APPLE LOCAL link optimizations 6499452 */"			   \
+       %:version-compare(>< 10.5 10.6 mmacosx-version-min= -lgcc_s.10.5)   \
        -lgcc}"
 
 /* We specify crt0.o as -lcrt0.o so that ld will search the library path.
@@ -456,7 +523,8 @@ do {					\
 #undef  STARTFILE_SPEC
 #define STARTFILE_SPEC							    \
   "%{Zdynamiclib: %(darwin_dylib1) }					    \
-   %{!Zdynamiclib:%{Zbundle:%{!static:-lbundle1.o}}			    \
+   "/* APPLE LOCAL link optimizations 6499452 */"			    \
+   %{!Zdynamiclib:%{Zbundle:%{!static: %(darwin_bundle1)}}		    \
      %{!Zbundle:%{pg:%{static:-lgcrt0.o}				    \
                      %{!static:%{object:-lgcrt0.o}			    \
                                %{!object:%{preload:-lgcrt0.o}		    \
@@ -466,7 +534,9 @@ do {					\
                                 %{!object:%{preload:-lcrt0.o}		    \
                                   %{!preload: %(darwin_crt1)		    \
 					      %(darwin_crt2)}}}}}}	    \
-  %{shared-libgcc:%:version-compare(< 10.5 mmacosx-version-min= crt3.o%s)}"
+  %{shared-libgcc:							    \
+    %{!miphoneos-version-min=*:						    \
+      %:version-compare(< 10.5 mmacosx-version-min= crt3.o%s)}}"
 /* APPLE LOCAL end mainline  */
 
 /* The native Darwin linker doesn't necessarily place files in the order
@@ -478,42 +548,65 @@ do {					\
 #define DARWIN_EXTRA_SPECS						\
   { "darwin_crt1", DARWIN_CRT1_SPEC },					\
   { "darwin_dylib1", DARWIN_DYLIB1_SPEC },				\
+  /* APPLE LOCAL link optimizations 6499452 */				\
+  { "darwin_bundle1", DARWIN_BUNDLE1_SPEC },				\
   { "darwin_minversion", DARWIN_MINVERSION_SPEC },			\
 /* APPLE LOCAL end mainline */						\
-/* APPLE LOCAL begin ARM 5342595 */					\
-  { "darwin_dsymutil", DARWIN_DSYMUTIL_SPEC },
-/* APPLE LOCAL end ARM 5342595 */
- 
-/* APPLE LOCAL begin mainline */
+/* APPLE LOCAL begin ARM 5683689 */					\
+  { "darwin_cc1_minversion", DARWIN_CC1_MINVERSION_SPEC },		\
+  { "darwin_ld_minversion", DARWIN_LD_MINVERSION_SPEC },		\
+/* APPLE LOCAL end ARM 5683689 */					\
+/* APPLE LOCAL ARM 5681645 */						\
+  { "darwin_iphoneos_libgcc", DARWIN_IPHONEOS_LIBGCC_SPEC },
+
+/* APPLE LOCAL begin ARM 5683689 */
+/* APPLE LOCAL begin link optimizations 6999417 */
 #define DARWIN_DYLIB1_SPEC						\
-  "%:version-compare(!> 10.5 mmacosx-version-min= -ldylib1.o)		\
-   %:version-compare(>= 10.5 mmacosx-version-min= -ldylib1.10.5.o)"
+  "%{miphoneos-version-min=*:						\
+    %:version-compare(< 3.1 miphoneos-version-min= -ldylib1.o)}		\
+   %{!miphoneos-version-min=*:						\
+     %:version-compare(!> 10.5 mmacosx-version-min= -ldylib1.o)		\
+     %:version-compare(>= 10.5 mmacosx-version-min= -ldylib1.10.5.o)}"
+
+/* APPLE LOCAL begin link optimizations 6499452 */
+#define DARWIN_BUNDLE1_SPEC						\
+  "%{miphoneos-version-min=*:						\
+    %:version-compare(< 3.1 miphoneos-version-min= -lbundle1.o)}	\
+   %{!miphoneos-version-min=*: -lbundle1.o }"
+/* APPLE LOCAL end link optimizations 6499452 */
 
 #define DARWIN_CRT1_SPEC						\
-  "%:version-compare(!> 10.5 mmacosx-version-min= -lcrt1.o)		\
-   %:version-compare(>= 10.5 mmacosx-version-min= -lcrt1.10.5.o)"
-/* APPLE LOCAL end mainline */
+/* APPLE LOCAL ARM 5823776 iphoneos should use crt1.o */		\
+  "%{miphoneos-version-min=*:						\
+    %:version-compare(< 3.1 miphoneos-version-min= -lcrt1.o)		\
+    %:version-compare(>= 3.1 miphoneos-version-min= -lcrt1.3.1.o)}	\
+   %{!miphoneos-version-min=*:						\
+     %:version-compare(!> 10.5 mmacosx-version-min= -lcrt1.o)		\
+     %:version-compare(>= 10.5 mmacosx-version-min= -lcrt1.10.5.o)}"
+/* APPLE LOCAL end link optimizations 6999417 */
+/* APPLE LOCAL end ARM 5683689 */
+
+/* APPLE LOCAL begin prefer -lSystem 6645902 */
+#define LINK_GCC_C_SEQUENCE_SPEC "%G %L"
+/* APPLE LOCAL end prefer -lSystem 6645902 */
 
 /* Default Darwin ASM_SPEC, very simple.  */
+/* APPLE LOCAL begin kext weak_import 5935650 */
 /* APPLE LOCAL begin radar 4161346 */
 #define ASM_SPEC "-arch %(darwin_arch) \
   %{Zforce_cpusubtype_ALL:-force_cpusubtype_ALL} \
-  %{!Zforce_cpusubtype_ALL:%{faltivec:-force_cpusubtype_ALL}}"
+  %{!Zforce_cpusubtype_ALL:%{faltivec:-force_cpusubtype_ALL}} \
+  %{mkernel|static|fapple-kext:%{!Zdynamic:-static}}"
 /* APPLE LOCAL end radar 4161346 */
+/* APPLE LOCAL end kext weak_import 5935650 */
 /* APPLE LOCAL begin mainline 4.3 2006-10-31 4370143 */
 /* We still allow output of STABS.  */
 
 #define DBX_DEBUGGING_INFO 1
 
-/* LLVM LOCAL begin */
-/* Prefer DWARF only if appropriate dsymutil is available.  */
-  #define DWARF2_DEBUGGING_INFO
-#ifdef HAVE_DSYMUTIL
-  #define PREFERRED_DEBUGGING_TYPE DWARF2_DEBUG
-#else
-  #define PREFERRED_DEBUGGING_TYPE DBX_DEBUG
-#endif
-/* LLVM LOCAL end */
+/* Prefer DWARF2.  */
+#define DWARF2_DEBUGGING_INFO
+#define PREFERRED_DEBUGGING_TYPE DWARF2_DEBUG
 
 /* APPLE LOCAL end mainline 4.3 2006-10-31 4370143 */
 #define DEBUG_FRAME_SECTION	"__DWARF,__debug_frame,regular,debug"
@@ -527,6 +620,8 @@ do {					\
 /* APPLE LOCAL begin pubtypes, approved for 4.3  4535968 */
 #define DEBUG_PUBTYPES_SECTION  "__DWARF,__debug_pubtypes,regular,debug"
 /* APPLE LOCAL end pubtypes, approved for 4.3 4535968 */
+/* APPLE LOCAL radar 6275985 debug inlined section */
+#define DEBUG_INLINED_SECTION   "__DWARF,__debug_inlined,regular,debug"
 #define DEBUG_STR_SECTION	"__DWARF,__debug_str,regular,debug"
 #define DEBUG_RANGES_SECTION	"__DWARF,__debug_ranges,regular,debug"
 
@@ -580,23 +675,55 @@ do {					\
   } while (0)
 
 /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
 /* As in the warning above, alias definitions aren't supported on Mach-O. */
 #define TARGET_DOES_NOT_SUPPORT_ALIAS_DEFINITIONS
 
-/* weak_import, a Darwin special, does not make function definitions weak. */
+/* weak_import, a Darwin special, does not make definitions weak. */
 #define TARGET_ADJUST_LLVM_LINKAGE(FN, DECL)                            \
   do {                                                                  \
     if ((DECL) &&                                                       \
-        TREE_CODE (DECL) == FUNCTION_DECL &&                            \
-        !DECL_EXTERNAL (DECL) &&                                        \
+        (TREE_CODE (DECL) != FUNCTION_DECL || !DECL_EXTERNAL (DECL)) && \
         TREE_PUBLIC (DECL) &&                                           \
         DECL_WEAK (DECL) &&                                             \
         ! lookup_attribute ("weak", DECL_ATTRIBUTES (DECL)) &&          \
 	lookup_attribute ("weak_import", DECL_ATTRIBUTES (DECL)) &&     \
         (FN)->hasWeakLinkage()) {                                       \
-      (FN)->setLinkage(Function::ExternalLinkage);                      \
+      (FN)->setLinkage(TREE_CODE(DECL)==FUNCTION_DECL ?                 \
+                       Function::ExternalLinkage :                      \
+                       GlobalValue::ExternalLinkage);                   \
     }                                                                   \
   } while (0)
+
+/* LLVM LOCAL - begin radar 6389998 */
+/* A const CFString is created as an anonymous global variable. LLVM then gives
+   it the name '__unnamed_#_#'. This causes troubles with the runtime, which
+   expects the name to be internal. Give it an internal name here.  */
+#define TARGET_ADJUST_CFSTRING_NAME(GV, SEC)                            \
+  do {                                                                  \
+    if (!GV->hasName() && GV->hasInternalLinkage() &&                   \
+        strcmp((SEC), "__DATA, __cfstring") == 0) {                     \
+      static unsigned i = 0;                                            \
+      const char *fmt = "\01L_unnamed_cfstring_%d";                     \
+      char *N = (char *)alloca(strlen(fmt) + 37);                       \
+      sprintf(N, fmt, i++);                                             \
+      GV->setName(N);                                                   \
+      GV->setAlignment(TARGET_64BIT ? 8 : 4);                           \
+    }                                                                   \
+  } while (0)
+/* LLVM LOCAL - end radar 6389998 */
+
+/* LLVM LOCAL - begin radar 7291825 */
+/* Give a constant string a sufficient alignment for the platform.  */
+#define TARGET_ADJUST_CSTRING_ALIGN(GV)                                 \
+  do {                                                                  \
+    if (GV->hasInternalLinkage()) {                                     \
+      GV->setAlignment(TARGET_64BIT ? 8 : 4);                           \
+    }                                                                   \
+  } while (0)
+/* LLVM LOCAL - end radar 7291825 */
+
+#endif
 /* LLVM LOCAL end */
 
 /* Darwin has the pthread routines in libSystem, which every program
@@ -690,7 +817,10 @@ do {					\
 #undef OBJC_FLAG_ZEROCOST_EXCEPTIONS
 #define OBJC_FLAG_ZEROCOST_EXCEPTIONS						\
   do {									\
-       if (strverscmp (darwin_macosx_version_min, "10.5") < 0) 		\
+       /* APPLE LOCAL begin ARM 5683689 */				\
+       if (darwin_macosx_version_min					\
+	   && strverscmp (darwin_macosx_version_min, "10.5") < 0)	\
+       /* APPLE LOCAL end ARM 5683689 */				\
 	 error ("Mac OS X version 10.5 or later is needed for zerocost-exceptions"); \
      } while (0)
 /* APPLE LOCAL end radar 5023725 */
@@ -718,6 +848,10 @@ do {					\
          } 								\
        if (flag_objc_abi == -1)						\
          flag_objc_abi = (flag_next_runtime && TARGET_64BIT) ? 2 : 1;	\
+       /* APPLE LOCAL begin ARM hybrid objc-2.0 */			\
+       if (flag_objc_legacy_dispatch == -1)				\
+	 flag_objc_legacy_dispatch = (flag_objc_abi < 2);		\
+       /* APPLE LOCAL end ARM hybrid objc-2.0 */			\
          /* APPLE LOCAL begin radar 2848255 */                          \
         /* APPLE LOCAL begin radar 5023725 */                           \
         if (flag_objc_abi == 2)                                         \
@@ -734,19 +868,41 @@ do {					\
              flag_objc_abi = 2;  /* recover */                          \
            }                                                            \
          /* APPLE LOCAL end radar 2848255 */                            \
+	/* APPLE LOCAL begin 5660282 */					\
+	if (darwin_iphoneos_version_min && flag_objc_gc)		\
+	  {								\
+	    warning (0, "-fobjc-gc not supported for iPhone OS; ignoring.");\
+	    flag_objc_gc = 0;						\
+	  }								\
+	if (darwin_iphoneos_version_min && flag_objc_gc_only)		\
+	  {								\
+	    warning (0, "-fobjc-gc-only not supported for iPhone OS; ignoring.");\
+	    flag_objc_gc_only = 0;					\
+	  }								\
+	/* APPLE LOCAL end 5660282 */					\
   } while (0)
 /* APPLE LOCAL end radar 4862848 */
 
 /* APPLE LOCAL begin radar 4531086 */
 #undef OBJC_WARN_OBJC2_FEATURES
 #define OBJC_WARN_OBJC2_FEATURES(MESSAGE)				\
+  /* APPLE LOCAL begin ARM 5683689 */					\
   do {									\
-    if (strverscmp (darwin_macosx_version_min, "10.5") < 0)		\
+    if (darwin_macosx_version_min					\
+	&& strverscmp (darwin_macosx_version_min, "10.5") < 0)		\
+  /* APPLE LOCAL end ARM 5683689 */					\
       warning (0, "Mac OS X version 10.5 or later is needed for use of %s", \
 	       MESSAGE);						\
   } while (0)
 /* APPLE LOCAL end radar 4531086 */
 
+/* APPLE LOCAL begin radar 6307941 */
+#undef OBJC2_ABI_DISPATCH
+#define OBJC2_ABI_DISPATCH						\
+(darwin_macosx_version_min						\
+ && strverscmp (darwin_macosx_version_min, "10.6") < 0)
+/* APPLE LOCAL end radar 6307941 */
+
 /* The RTTI data (e.g., __ti4name) is common and public (and static),
    but it does need to be referenced via indirect PIC data pointers.
    The machopic_define_symbol calls are telling the machopic subsystem
@@ -842,7 +998,8 @@ do {					\
        else if (!strncmp (xname, ".objc_class_name_", 17))		     \
 	 fprintf (FILE, "%s", xname);					     \
        else if (xname[0] != '"' && name_needs_quotes (xname))		     \
-	 fprintf (FILE, "\"%s\"", xname);				     \
+	 /* APPLE LOCAL 5782111 */					     \
+	 asm_fprintf (FILE, "\"%U%s\"", xname);				     \
        else								     \
          asm_fprintf (FILE, "%U%s", xname);				     \
   } while (0)
@@ -935,6 +1092,7 @@ extern GTY(()) section * darwin_sections[NUM_DARWIN_SECTIONS];
        } while (0)
 #else
 /* LLVM LOCAL end */
+
 #define ASM_DECLARE_UNRESOLVED_REFERENCE(FILE,NAME)			\
     do {								\
 	 if (FILE) {							\
@@ -949,6 +1107,20 @@ extern GTY(()) section * darwin_sections[NUM_DARWIN_SECTIONS];
 /* LLVM LOCAL */
 #endif /*ENABLE_LLVM*/
 
+/* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
+#define ASM_DECLARE_CLASS_REFERENCE(FILE,NAME)                          \
+  do {                                                                  \
+    if (FILE) {                                                         \
+      char *Buffer = alloca(strlen(NAME) + 30);                         \
+      sprintf(Buffer, "\t%s=0", NAME);                                  \
+      llvm_emit_file_scope_asm(Buffer);                                 \
+      sprintf(Buffer, "\t.globl %s", NAME);                             \
+      llvm_emit_file_scope_asm(Buffer);                                 \
+    }                                                                   \
+  } while (0)
+#else
+/* LLVM LOCAL end */
 #define ASM_DECLARE_CLASS_REFERENCE(FILE,NAME)				\
     do {								\
 	 if (FILE) {							\
@@ -958,6 +1130,8 @@ extern GTY(()) section * darwin_sections[NUM_DARWIN_SECTIONS];
 	   (*targetm.asm_out.globalize_label) (FILE, NAME);		\
 	 }								\
        } while (0)
+/* LLVM LOCAL */
+#endif /*ENABLE_LLVM*/
 
 /* Globalizing directive for a label.  */
 #define GLOBAL_ASM_OP ".globl "
@@ -983,6 +1157,8 @@ extern GTY(()) section * darwin_sections[NUM_DARWIN_SECTIONS];
     darwin_handle_kext_attribute },					     \
   /* APPLE LOCAL ObjC GC */						     \
   { "objc_gc", 1, 1, false, true, false, darwin_handle_objc_gc_attribute },  \
+  /* APPLE LOCAL radar 5595352 */					     \
+  { "NSObject", 0, 0, false, true, false, darwin_handle_nsobject_attribute },\
   { "weak_import", 0, 0, true, false, false,				     \
     darwin_handle_weak_import_attribute }
 
@@ -1021,8 +1197,11 @@ enum machopic_addr_class {
 
 /* Macros defining the various PIC cases.  */
 
-/* APPLE LOCAL mdynamic-no-pic */
+/* APPLE LOCAL begin mdynamic-no-pic */
+/* LLVM LOCAL - possibly redefine MACHO_DYNAMIC_NO_PIC_P */
+#undef MACHO_DYNAMIC_NO_PIC_P
 #define MACHO_DYNAMIC_NO_PIC_P	(TARGET_MACHO_DYNAMIC_NO_PIC)
+/* APPLE LOCAL end mdynamic-no-pic */
 /* APPLE LOCAL begin mach-o cleanup */
 #undef MACHOPIC_INDIRECT
 #define MACHOPIC_INDIRECT	(flag_pic || MACHO_DYNAMIC_NO_PIC_P)
@@ -1064,32 +1243,40 @@ enum machopic_addr_class {
       }								\
   } while (0)
 
+/* APPLE LOCAL begin ARM 5603763 */
+/* Given a symbol name, remove quotes, prefix it with "L", suffix it
+   with SUFFIX, and re-apply quotes if needed.  */
+
+#define GEN_SUFFIXED_NAME_FOR_SYMBOL(BUF,SYMBOL,SYMBOL_LENGTH,SUFFIX)	\
+  do {									\
+    const char *symbol_ = (SYMBOL);					\
+    char *buffer_ = (BUF);						\
+    if (symbol_[0] == '"')						\
+      {									\
+        strcpy (buffer_, "\"L");					\
+        strcpy (buffer_ + 2, symbol_ + 1);				\
+	strcpy (buffer_ + (SYMBOL_LENGTH), SUFFIX "\"");		\
+      }									\
+    else if (name_needs_quotes (symbol_))				\
+      {									\
+        strcpy (buffer_, "\"L");					\
+        strcpy (buffer_ + 2, symbol_);					\
+	strcpy (buffer_ + (SYMBOL_LENGTH) + 2, SUFFIX "\"");		\
+      }									\
+    else								\
+      {									\
+        strcpy (buffer_, "L");						\
+        strcpy (buffer_ + 1, symbol_);					\
+	strcpy (buffer_ + (SYMBOL_LENGTH) + 1, SUFFIX);			\
+      }									\
+  } while (0)
+
 /* Given a symbol name string, create the lazy pointer version
    of the symbol name.  */
 
 #define GEN_LAZY_PTR_NAME_FOR_SYMBOL(BUF,SYMBOL,SYMBOL_LENGTH)	\
-  do {								\
-    const char *symbol_ = (SYMBOL);                             \
-    char *buffer_ = (BUF);					\
-    if (symbol_[0] == '"')					\
-      {								\
-        strcpy (buffer_, "\"L");				\
-        strcpy (buffer_ + 2, symbol_ + 1);			\
-	strcpy (buffer_ + (SYMBOL_LENGTH), "$lazy_ptr\"");	\
-      }								\
-    else if (name_needs_quotes (symbol_))			\
-      {								\
-        strcpy (buffer_, "\"L");				\
-        strcpy (buffer_ + 2, symbol_);				\
-	strcpy (buffer_ + (SYMBOL_LENGTH) + 2, "$lazy_ptr\"");	\
-      }								\
-    else							\
-      {								\
-        strcpy (buffer_, "L");					\
-        strcpy (buffer_ + 1, symbol_);				\
-	strcpy (buffer_ + (SYMBOL_LENGTH) + 1, "$lazy_ptr");	\
-      }								\
-  } while (0)
+  GEN_SUFFIXED_NAME_FOR_SYMBOL (BUF, SYMBOL, SYMBOL_LENGTH, "$lazy_ptr")
+/* APPLE LOCAL end ARM 5603763 */
 
 #define EH_FRAME_SECTION_NAME   "__TEXT"
 #define EH_FRAME_SECTION_ATTR ",coalesced,no_toc+strip_static_syms+live_support"
@@ -1099,7 +1286,8 @@ enum machopic_addr_class {
 
 #undef ASM_PREFERRED_EH_DATA_FORMAT
 #define ASM_PREFERRED_EH_DATA_FORMAT(CODE,GLOBAL)  \
-  (((CODE) == 2 && (GLOBAL) == 1) \
+  /* APPLE LOCAL EH __TEXT __gcc_except_tab 5819051 */	      \
+  ((((CODE) == 2 || (CODE) == 0) && (GLOBAL) == 1)	      \
    ? (DW_EH_PE_pcrel | DW_EH_PE_indirect | DW_EH_PE_sdata4) : \
      ((CODE) == 1 || (GLOBAL) == 0) ? DW_EH_PE_pcrel : DW_EH_PE_absptr)
 
@@ -1140,16 +1328,17 @@ enum machopic_addr_class {
 #define HANDLE_PRAGMA_PACK_PUSH_POP 1
 
 /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
 /* Handle pragma pack separately */
 #define TARGET_OVERRIDE_PRAGMA_PACK_HANDLER 1
+#endif
 /* LLVM LOCAL end */
 
 #define DARWIN_REGISTER_TARGET_PRAGMAS()			\
   do {								\
-    /* APPLE LOCAL begin mainline 2007-10-10 5497482 */		\
-    cpp_register_pragma (parse_in, NULL, "mark",		\
-			 darwin_pragma_ignore, false);		\
-    /* APPLE LOCAL end mainline 2007-10-10 5497482 */		\
+    /* APPLE LOCAL begin pragma mark 5614511 */			\
+    /* Removed mark.  */					\
+    /* APPLE LOCAL end pragma mark 5614511 */			\
     c_register_pragma (0, "options", darwin_pragma_options);	\
     c_register_pragma (0, "segment", darwin_pragma_ignore);	\
     /* APPLE LOCAL pragma fenv */                               \
@@ -1219,15 +1408,24 @@ extern void abort_assembly_and_exit (int status) ATTRIBUTE_NORETURN;
 /* APPLE LOCAL begin zerofill 20020218 --turly  */
 /* This keeps uninitialized data from bloating the data when -fno-common.
    Radar 2863107.  */
-#define ASM_OUTPUT_ZEROFILL(FILE, NAME, SIZE, ALIGNMENT)		    \
-  do {									    \
-    unsigned HOST_WIDE_INT _new_size = SIZE;		    		    \
-    if (_new_size == 0) _new_size = 1;					    \
-    fputs (".zerofill __DATA, __common, ", (FILE));			    \
-    assemble_name ((FILE), (NAME));					    \
-    fprintf ((FILE), ", " HOST_WIDE_INT_PRINT_DEC, _new_size); 		    \
-    fprintf ((FILE), ", " HOST_WIDE_INT_PRINT_DEC "\n",			    \
-	     (HOST_WIDE_INT) (ALIGNMENT));				    \
+#define ASM_OUTPUT_ZEROFILL(FILE, NAME, SECT, SIZE, ALIGNMENT)	\
+  do {								\
+    section *darwin_sect = SECT;				\
+								\
+    unsigned HOST_WIDE_INT _new_size = SIZE;			\
+    if (_new_size == 0) _new_size = 1;				\
+    fputs (".zerofill ", (FILE));				\
+    if (darwin_sect->common.flags & SECTION_NAMED)		\
+      {								\
+	fputs (darwin_sect->named.name, (FILE));		\
+	fputs (", ", (FILE));					\
+      }								\
+    else							\
+      fputs ("__DATA, __common, ", (FILE));			\
+    assemble_name ((FILE), (NAME));				\
+    fprintf ((FILE), ", " HOST_WIDE_INT_PRINT_DEC, _new_size);	\
+    fprintf ((FILE), ", " HOST_WIDE_INT_PRINT_DEC "\n",		\
+	     (HOST_WIDE_INT) (ALIGNMENT));			\
   } while (0)
 /* APPLE LOCAL end zerofill 20020218 --turly  */
 
@@ -1261,7 +1459,6 @@ void add_framework_path (char *);
   darwin_handle_c_option (CODE, ARG, VALUE)
 /* APPLE LOCAL end iframework for 4.3 4094959 */
 
-
 /* LLVM LOCAL begin */
 #ifdef ENABLE_LLVM
 /* LLVM_IMPLICIT_TARGET_GLOBAL_VAR_SECTION - Given a VAR_DECL for a global
@@ -1274,12 +1471,17 @@ void add_framework_path (char *);
  */
 #define LLVM_IMPLICIT_TARGET_GLOBAL_VAR_SECTION(decl)                   \
   (((DECL_NAME (decl) &&                                                \
-    TREE_CODE (DECL_NAME (decl)) == IDENTIFIER_NODE &&                  \
-    IDENTIFIER_POINTER (DECL_NAME (decl)) &&                            \
-    !strncmp (IDENTIFIER_POINTER (DECL_NAME (decl)), "L_OBJC_", 7)) ||  \
-   TREE_CODE(decl) == CONST_DECL) ?                                     \
-     darwin_objc_llvm_implicit_target_global_var_section(decl) : 0)
+     TREE_CODE (DECL_NAME (decl)) == IDENTIFIER_NODE &&                 \
+     IDENTIFIER_POINTER (DECL_NAME (decl)) &&                           \
+     (!strncmp (IDENTIFIER_POINTER (DECL_NAME (decl)), "_OBJC_", 6) ||  \
+      !strncmp (IDENTIFIER_POINTER (DECL_NAME (decl)), "OBJC_", 5) ||   \
+      !strncmp (IDENTIFIER_POINTER (DECL_NAME (decl)), "L_OBJC_", 7) || \
+      !strncmp (IDENTIFIER_POINTER (DECL_NAME (decl)), "l_OBJC_", 7) || \
+      !strncmp (IDENTIFIER_POINTER (DECL_NAME (decl)), "l_objc_", 7))) || \
+    TREE_CODE(decl) == CONST_DECL) ?                                    \
+   darwin_objc_llvm_implicit_target_global_var_section(decl) : 0)
 const char *darwin_objc_llvm_implicit_target_global_var_section(tree);
+const char *darwin_objc_llvm_special_name_section(const char*);
 
 /* Darwin X86-64 only supports PIC code generation. */
 #if defined (TARGET_386)
@@ -1288,17 +1490,30 @@ const char *darwin_objc_llvm_implicit_target_global_var_section(tree);
     argvec.push_back ("--relocation-model=pic");     \
   else if (!MACHO_DYNAMIC_NO_PIC_P)                  \
     argvec.push_back ("--relocation-model=static")
-#else /* defined (TARGET_386) */
+#elif defined (TARGET_ARM)
+#define LLVM_SET_TARGET_OPTIONS(argvec)              \
+  if (flag_pic)                                      \
+    argvec.push_back ("--relocation-model=pic");     \
+  else if (!MACHO_DYNAMIC_NO_PIC_P)                  \
+    argvec.push_back ("--relocation-model=static");  \
+  if (darwin_iphoneos_version_min)                   \
+    {                                                \
+      const char *p = darwin_iphoneos_version_min;   \
+      if (ISDIGIT (*p) && *p == '1' || *p == '2')    \
+        {                                            \
+          ++p;                                       \
+          if (!p || *p == '.')                       \
+            argvec.push_back("--arm-reserve-r9");    \
+        }                                            \
+    }
+#else /* !TARGET_386 && !TARGET_ARM */
 #define LLVM_SET_TARGET_OPTIONS(argvec)              \
   if (flag_pic)                                      \
     argvec.push_back ("--relocation-model=pic");     \
   else if (!MACHO_DYNAMIC_NO_PIC_P)                  \
     argvec.push_back ("--relocation-model=static")
-#endif /* defined (TARGET_386) */
+#endif /* !TARGET_386 && !TARGET_ARM */
 
-/* On Darwin _Unwind_Resume is sensitive to the dynamic stack layout; we
-   use _Unwind_Resume_or_Rethrow instead.  */
-#define LLVM_STACKSENSITIVE_UNWIND_RESUME 1
 #endif
 /* LLVM LOCAL end */
 
@@ -1398,12 +1613,40 @@ extern int flag_apple_kext;
 
 /* APPLE LOCAL begin mainline 2007-06-14 5235474 */
 #ifndef CROSS_DIRECTORY_STRUCTURE
-extern void darwin_default_min_version (int * argc, char *** argv);
+/* APPLE LOCAL begin ARM 5683689 */
+extern void darwin_default_min_version (int * argc, char *** argv,
+					enum darwin_version_type vers_type);
 #define GCC_DRIVER_HOST_INITIALIZATION \
   /* APPLE LOCAL isysroot 5083137 */ \
   GCC_DRIVER_HOST_INITIALIZATION1; \
-  darwin_default_min_version (&argc, &argv)
+  darwin_default_min_version (&argc, &argv, DARWIN_DEFAULT_VERSION_TYPE)
+/* APPLE LOCAL end ARM 5683689 */
 #endif /* CROSS_DIRECTORY_STRUCTURE */
 /* APPLE LOCAL end mainline 2007-06-14 5235474 */
 
+/* LLVM LOCAL begin sysroot */
+char *darwin_build_sysroot_path(const char *sysroot, const char *path);
+#undef TARGET_BUILD_SYSROOT_PATH
+#define TARGET_BUILD_SYSROOT_PATH(S,P) darwin_build_sysroot_path((S), (P))
+/* LLVM LOCAL end sysroot */
+
+/* APPLE LOCAL begin radar 6230142 */
+/* libgcc2 is strict c90; bool doesn't work. */
+extern unsigned darwin_llvm_override_target_version(const char*, char**);
+#define LLVM_OVERRIDE_TARGET_VERSION(T,N)        \
+  darwin_llvm_override_target_version(T,N)
+/* APPLE LOCAL end radar 6230142 */
+
+/* LLVM LOCAL begin */
+#ifdef WARN_FORMAT_INIT
+#undef WARN_FORMAT_INIT
+#endif
+#define WARN_FORMAT_INIT 1
+
+#ifdef WARN_FORMAT_SECURITY_INIT
+#undef WARN_FORMAT_SECURITY_INIT
+#endif
+#define WARN_FORMAT_SECURITY_INIT 1
+/* LLVM LOCAL end */
+
 #endif /* CONFIG_DARWIN_H */
