@@ -159,6 +159,9 @@ make_thunk (tree function, bool this_adjusting,
   DECL_PENDING_INLINE_P (thunk) = 0;
   DECL_INLINE (thunk) = 0;
   DECL_DECLARED_INLINE_P (thunk) = 0;
+  /* LLVM LOCAL begin inlinehint attribute */
+  DECL_EXPLICIT_INLINE_P (thunk) = 0;
+  /* LLVM LOCAL end inlinehint attribute */
   /* Nor has it been deferred.  */
   DECL_DEFERRED_FN (thunk) = 0;
   /* Nor is it a template instantiation.  */
@@ -283,6 +286,9 @@ make_alias_for (tree function, tree newid)
   DECL_PENDING_INLINE_P (alias) = 0;
   DECL_INLINE (alias) = 0;
   DECL_DECLARED_INLINE_P (alias) = 0;
+  /* LLVM LOCAL begin inlinehint attribute */
+  DECL_EXPLICIT_INLINE_P (alias) = 0;
+  /* LLVM LOCAL end inlinehint attribute */
   DECL_DEFERRED_FN (alias) = 0;
   DECL_USE_TEMPLATE (alias) = 0;
   DECL_TEMPLATE_INSTANTIATED (alias) = 0;
@@ -354,14 +360,7 @@ use_thunk (tree thunk_fndecl, bool emit_p)
   if (!emit_p)
     return;
 
-  /* LLVM LOCAL begin */
-  /* PR843 */
-#ifndef ENABLE_LLVM
   if (TARGET_USE_LOCAL_THUNK_ALIAS_P (function))
-#else
-  if (0 && TARGET_USE_LOCAL_THUNK_ALIAS_P (function))
-#endif
-  /* LLVM LOCAL end */
    alias = make_alias_for_thunk (function);
   else
    alias = function;
@@ -1118,6 +1117,16 @@ implicitly_declare_fn (special_function_kind kind, tree type, bool const_p)
   TREE_CHAIN (this_parm) = DECL_ARGUMENTS (fn);
   DECL_ARGUMENTS (fn) = this_parm;
 
+  /* APPLE LOCAL begin mainline aligned functions 5933878 */
+  /* If pointers to member functions use the least significant bit to
+     indicate whether a function is virtual, ensure a pointer
+     to this function will have that bit clear.  */
+  if (TARGET_PTRMEMFUNC_VBIT_LOCATION == ptrmemfunc_vbit_in_pfn
+      && TREE_CODE (fn_type) == METHOD_TYPE
+      && DECL_ALIGN (fn) < 2 * BITS_PER_UNIT)
+    DECL_ALIGN (fn) = 2 * BITS_PER_UNIT;
+  /* APPLE LOCAL end mainline aligned functions 5933878 */
+
   grokclassfn (type, fn, kind == sfk_destructor ? DTOR_FLAG : NO_SPECIAL);
   set_linkage_according_to_type (type, fn);
   rest_of_decl_compilation (fn, toplevel_bindings_p (), at_eof);
