@@ -2091,7 +2091,8 @@ getpagesize (void)
 #endif
 }
 
-#ifdef __i386__
+/* LLVM LOCAL */
+#if defined(__i386__) && ! defined(_WINBASE_H)
 extern int VirtualProtect (char *, int, int, int *) __attribute__((stdcall));
 #endif
 
@@ -2126,7 +2127,8 @@ TRANSFER_FROM_TRAMPOLINE
 #endif
 #endif /* L_trampoline */
 
-#ifndef __CYGWIN__
+/* LLVM LOCAL */
+#if !defined (__CYGWIN__) && !defined(__MINGW32__)
 #ifdef L__main
 
 #include "gbl-ctors.h"
