@@ -32,22 +32,21 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <cassert>
 #include <map>
 #include <string>
+#include "llvm/CallingConv.h"
 #include "llvm/Intrinsics.h"
 #include "llvm/ADT/IndexedMap.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/ADT/SetVector.h"
-#include "llvm/Support/DataTypes.h"
-#include "llvm/Support/LLVMBuilder.h"
-#include "llvm/Support/Streams.h"
+#include "llvm/System/DataTypes.h"
+#include "llvm/Support/IRBuilder.h"
+#include "llvm/Support/MathExtras.h"
+#include "llvm/Support/TargetFolder.h"
+#include "llvm/Support/raw_os_ostream.h"
 
 extern "C" {
 #include "llvm.h"
 }
 
-/// Internal gcc structure describing an exception handling region.  Declared
-/// here to avoid including all of except.h.
-struct eh_region;
-
 namespace llvm {
   class Module;
   class GlobalVariable;
@@ -68,6 +67,9 @@ namespace llvm {
 }
 using namespace llvm;
 
+typedef IRBuilder<true, TargetFolder> LLVMBuilder;
+typedef SmallPtrSet<union tree_node *, 16> treeset;
+
 /// TheModule - This is the current global module that we are compiling into.
 ///
 extern llvm::Module *TheModule;
@@ -80,43 +82,38 @@ extern llvm::DebugInfo *TheDebugInfo;
 ///
 extern llvm::TargetMachine *TheTarget;
 
+/// TheFolder - The constant folder to use.
+extern TargetFolder *TheFolder;
+
 /// getTargetData - Return the current TargetData object from TheTarget.
 const TargetData &getTargetData();
 
-/// AsmOutFile - A C++ ostream wrapper around asm_out_file.
-///
-extern llvm::OStream *AsmOutFile;
-
-/// StaticCtors/StaticDtors - The static constructors and destructors that we
-/// need to emit.
-extern std::vector<std::pair<Function*, int> > StaticCtors, StaticDtors;
-
 /// AttributeUsedGlobals - The list of globals that are marked attribute(used).
 extern SmallSetVector<Constant *,32> AttributeUsedGlobals;
 
-/// AttributeNoinlineFunctions - The list of functions that are 
-/// marked attribute(noinline)
-extern std::vector<Constant*> AttributeNoinlineFunctions;
-
 extern Constant* ConvertMetadataStringToGV(const char* str);
 
 /// AddAnnotateAttrsToGlobal - Adds decls that have a
 /// annotate attribute to a vector to be emitted later.
 extern void AddAnnotateAttrsToGlobal(GlobalValue *GV, union tree_node* decl);
 
-void changeLLVMValue(Value *Old, Value *New);
+void changeLLVMConstant(Constant *Old, Constant *New);
 void readLLVMTypesStringTable();
 void writeLLVMTypesStringTable();
 void readLLVMValues();
 void writeLLVMValues();
+void readLLVMTypeUsers();
+void writeLLVMTypeUsers();
+void eraseLocalLLVMValues();
 void clearTargetBuiltinCache();
 const char* extractRegisterName(union tree_node*);
+void handleVisibility(union tree_node* decl, GlobalValue *GV);
 
 struct StructTypeConversionInfo;
 
 /// Return true if and only if field no. N from struct type T is a padding
 /// element added to match llvm struct type size and gcc struct type size.
-bool isPaddingElement(const Type *T, unsigned N);
+bool isPaddingElement(union tree_node*, unsigned N);
 
 /// TypeConverter - Implement the converter from GCC types to LLVM types.
 ///
@@ -126,24 +123,17 @@ class TypeConverter {
   bool ConvertingStruct;
   
   /// PointersToReresolve - When ConvertingStruct is true, we handling of
-  /// POINTER_TYPE and REFERENCE_TYPE is changed to return opaque*'s instead of
-  /// recursively calling ConvertType.  When this happens, we add the
-  /// POINTER_TYPE to this list.
+  /// POINTER_TYPE, REFERENCE_TYPE, and BLOCK_POINTER_TYPE is changed to return
+  /// opaque*'s instead of recursively calling ConvertType.  When this happens,
+  /// we add the POINTER_TYPE to this list.
   ///
   std::vector<tree_node*> PointersToReresolve;
 
-  /// FieldIndexMap - Holds the mapping from a FIELD_DECL to the index of the
-  /// corresponding LLVM field.
-  std::map<tree_node *, unsigned int> FieldIndexMap;
 public:
   TypeConverter() : ConvertingStruct(false) {}
   
   const Type *ConvertType(tree_node *type);
 
-  /// GetFieldIndex - Returns the index of the LLVM field corresponding to
-  /// this FIELD_DECL.
-  unsigned int GetFieldIndex(tree_node *field_decl);
-
   /// GCCTypeOverlapsWithLLVMTypePadding - Return true if the specified GCC type
   /// has any data that overlaps with structure padding in the specified LLVM
   /// type.
@@ -156,24 +146,24 @@ public:
   const FunctionType *ConvertFunctionType(tree_node *type,
                                           tree_node *decl,
                                           tree_node *static_chain,
-                                          unsigned &CallingConv,
-                                          const ParamAttrsList *&PAL);
+                                          CallingConv::ID &CallingConv,
+                                          AttrListPtr &PAL);
   
   /// ConvertArgListToFnType - Given a DECL_ARGUMENTS list on an GCC tree,
   /// return the LLVM type corresponding to the function.  This is useful for
   /// turning "T foo(...)" functions into "T foo(void)" functions.
-  const FunctionType *ConvertArgListToFnType(tree_node *retty,
+  const FunctionType *ConvertArgListToFnType(tree_node *type,
                                              tree_node *arglist,
                                              tree_node *static_chain,
-                                             unsigned &CallingConv,
-                                             const ParamAttrsList *&PAL);
+                                             CallingConv::ID &CallingConv,
+                                             AttrListPtr &PAL);
   
 private:
   const Type *ConvertRECORD(tree_node *type, tree_node *orig_type);
   const Type *ConvertUNION(tree_node *type, tree_node *orig_type);
-  void SetFieldIndex(tree_node *field_decl, unsigned int Index);
-  void DecodeStructFields(tree_node *Field, StructTypeConversionInfo &Info);
+  bool DecodeStructFields(tree_node *Field, StructTypeConversionInfo &Info);
   void DecodeStructBitField(tree_node *Field, StructTypeConversionInfo &Info);
+  void SelectUnionMember(tree_node *type, StructTypeConversionInfo &Info);
 };
 
 extern TypeConverter *TheTypeConverter;
@@ -184,11 +174,9 @@ inline const Type *ConvertType(tree_node *type) {
   return TheTypeConverter->ConvertType(type);
 }
 
-/// GetFieldIndex - Given FIELD_DECL obtain its index.
+/// getINTEGER_CSTVal - Return the specified INTEGER_CST value as a uint64_t.
 ///
-inline unsigned int GetFieldIndex(tree_node *field_decl) {
-  return TheTypeConverter->GetFieldIndex(field_decl);
-}
+uint64_t getINTEGER_CSTVal(tree_node *exp);
 
 /// isInt64 - Return true if t is an INTEGER_CST that fits in a 64 bit integer.
 /// If Unsigned is false, returns whether it fits in a int64_t.  If Unsigned is
@@ -209,13 +197,10 @@ bool isPassedByInvisibleReference(tree_node *type);
 
 /// isSequentialCompatible - Return true if the specified gcc array or pointer
 /// type and the corresponding LLVM SequentialType lay out their components
-/// identically in memory.
+/// identically in memory, so doing a GEP accesses the right memory location.
+/// We assume that objects without a known size do not.
 bool isSequentialCompatible(tree_node *type);
 
-/// isArrayCompatible - Return true if the specified gcc array or pointer type
-/// corresponds to an LLVM array type.
-bool isArrayCompatible(tree_node *type);
-
 /// isBitfield - Returns whether to treat the specified field as a bitfield.
 bool isBitfield(tree_node *field_decl);
 
@@ -232,35 +217,56 @@ bool ValidateRegisterVariable(tree_node *decl);
 /// a pointer to the memory, its alignment and whether the access is volatile.
 struct MemRef {
   Value *Ptr;
-  unsigned Alignment;
   bool Volatile;
+private:
+  unsigned char LogAlign;
 
-  MemRef() : Ptr(0), Alignment(0), Volatile(false) {}
-  MemRef(Value *P, unsigned A, bool V)
-    : Ptr(P), Alignment(A), Volatile(V) {
-      // Allowing alignment 0 would complicate calculations, so forbid it.
-      assert(A && !(A & (A - 1)) && "Alignment not a power of 2!");
+public:
+  MemRef() : Ptr(0), Volatile(false), LogAlign(0) {}
+  MemRef(Value *P, uint32_t A, bool V) : Ptr(P), Volatile(V) {
+    // Forbid alignment 0 along with non-power-of-2 alignment values.
+    assert(isPowerOf2_32(A) && "Alignment not a power of 2!");
+    LogAlign = Log2_32(A);
+  }
+
+  uint32_t getAlignment() const {
+    return 1U << LogAlign;
   }
 };
 
 /// LValue - This struct represents an lvalue in the program.  In particular,
-/// the Ptr member indicates the memory that the lvalue lives in.  If this is
-/// a bitfield reference, BitStart indicates the first bit in the memory that
-/// is part of the field and BitSize indicates the extent.
+/// the Ptr member indicates the memory that the lvalue lives in.  Alignment
+/// is the alignment of the memory (in bytes).If this is a bitfield reference,
+/// BitStart indicates the first bit in the memory that is part of the field
+/// and BitSize indicates the extent.
 ///
 /// "LValue" is intended to be a light-weight object passed around by-value.
 struct LValue {
   Value *Ptr;
   unsigned char BitStart;
   unsigned char BitSize;
-  
-  LValue(Value *P) : Ptr(P), BitStart(255), BitSize(255) {}
-  LValue(Value *P, unsigned BSt, unsigned BSi) 
-    : Ptr(P), BitStart(BSt), BitSize(BSi) {
-      assert(BitStart == BSt && BitSize == BSi &&
-             "Bit values larger than 256?");
+private:
+  unsigned char LogAlign;
+
+public:
+  LValue() : Ptr(0), BitStart(255), BitSize(255), LogAlign(0) {}
+  LValue(Value *P, uint32_t A) : Ptr(P), BitStart(255), BitSize(255) {
+    // Forbid alignment 0 along with non-power-of-2 alignment values.
+    assert(isPowerOf2_32(A) && "Alignment not a power of 2!");
+    LogAlign = Log2_32(A);
+  }
+  LValue(Value *P, uint32_t A, unsigned BSt, unsigned BSi)
+  : Ptr(P), BitStart(BSt), BitSize(BSi) {
+    assert(BitStart == BSt && BitSize == BSi &&
+           "Bit values larger than 256?");
+    // Forbid alignment 0 along with non-power-of-2 alignment values.
+    assert(isPowerOf2_32(A) && "Alignment not a power of 2!");
+    LogAlign = Log2_32(A);
+  }
+
+  uint32_t getAlignment() const {
+    return 1U << LogAlign;
   }
-  
   bool isBitfield() const { return BitStart != 255; }
 };
 
@@ -274,7 +280,9 @@ class TreeToLLVM {
   Function *Fn;
   BasicBlock *ReturnBB;
   BasicBlock *UnwindBB;
-  BasicBlock *NoUnwindBB;
+  unsigned ReturnOffset;
+  // Lexical BLOCKS that we have previously seen and processed.
+  treeset SeenBlocks;
 
   // State that changes as the function is emitted.
 
@@ -286,6 +294,10 @@ class TreeToLLVM {
   // and managed by CreateTemporary.
   Instruction *AllocaInsertionPoint;
   
+  /// UniquedValues - Values defined using a no-op bitcast in order to make them
+  /// unique.  These can be simplified once the function has been emitted.
+  std::vector<BitCastInst *> UniquedValues;
+
   //===---------------------- Exception Handling --------------------------===//
 
   /// LandingPads - The landing pad for a given EH region.
@@ -310,22 +322,6 @@ class TreeToLLVM {
   /// FuncEHGetTypeID - Function used to return type id for give typeinfo.
   Function *FuncEHGetTypeID;
 
-  /// NumAddressTakenBlocks - Count the number of labels whose addresses are
-  /// taken.
-  uint64_t NumAddressTakenBlocks;
-
-  /// AddressTakenBBNumbers - For each label with its address taken, we keep 
-  /// track of its unique ID.
-  std::map<BasicBlock*, ConstantInt*> AddressTakenBBNumbers;
-  
-  /// IndirectGotoBlock - If non-null, the block that indirect goto's in this
-  /// function branch to.
-  BasicBlock *IndirectGotoBlock;
-  
-  /// IndirectGotoValue - This is set to be the alloca temporary that the
-  /// indirect goto block switches on.
-  Value *IndirectGotoValue;
-  
 public:
   TreeToLLVM(tree_node *fndecl);
   ~TreeToLLVM();
@@ -341,10 +337,6 @@ public:
   /// the address of the result.
   LValue EmitLV(tree_node *exp);
 
-  /// getIndirectGotoBlockNumber - Return the unique ID of the specified basic
-  /// block for uses that take the address of it.
-  Constant *getIndirectGotoBlockNumber(BasicBlock *BB);
-  
   /// getIndirectGotoBlock - Get (and potentially lazily create) the indirect
   /// goto block.
   BasicBlock *getIndirectGotoBlock();
@@ -386,8 +378,33 @@ public:
   /// instruction's type is a pointer to the specified type.
   AllocaInst *CreateTemporary(const Type *Ty);
 
+  /// CreateTempLoc - Like CreateTemporary, but returns a MemRef.
+  MemRef CreateTempLoc(const Type *Ty);
+
+  /// EmitAggregateCopy - Copy the elements from SrcLoc to DestLoc, using the
+  /// GCC type specified by GCCType to know which elements to copy.
+  void EmitAggregateCopy(MemRef DestLoc, MemRef SrcLoc, tree_node *GCCType);
+
+  // 'desired' and 'grand' are GCC BLOCK nodes, representing lexical
+  // blocks.  Assumes we're in the 'grand' context; push contexts
+  // until we reach the 'desired' context.
+  void push_regions(tree_node *desired, tree_node *grand);
+
+  // Given a GCC lexical context (BLOCK or FUNCTION_DECL), make it the
+  // new current BLOCK/context/scope.  Emit any local variables found
+  // in the new context.  Note that the variable emission order must be
+  // consistent with and without debug info; otherwise, the register
+  // allocation would change with -g, and users dislike that.
+  void switchLexicalBlock(tree_node *exp);
+
 private: // Helper functions.
 
+  // Walk over the lexical BLOCK() tree of the given FUNCTION_DECL;
+  // set the BLOCK_NUMBER() fields to the depth of each block, and
+  // add every var or type encountered in the BLOCK_VARS() lists to
+  // the given set.
+  void setLexicalBlockDepths(tree_node *t, treeset &s, unsigned level);
+
   /// StartFunctionBody - Start the emission of 'fndecl', outputing all
   /// declarations for parameters and setting things up.
   void StartFunctionBody();
@@ -406,19 +423,16 @@ private: // Helper functions.
   /// the previous block falls through into it, add an explicit branch.
   void EmitBlock(BasicBlock *BB);
   
-  /// EmitAggregateCopy - Copy the elements from SrcLoc to DestLoc, using the
-  /// GCC type specified by GCCType to know which elements to copy.
-  void EmitAggregateCopy(MemRef DestLoc, MemRef SrcLoc, tree_node *GCCType);
-
   /// EmitAggregateZero - Zero the elements of DestLoc.
   ///
   void EmitAggregateZero(MemRef DestLoc, tree_node *GCCType);
                          
   /// EmitMemCpy/EmitMemMove/EmitMemSet - Emit an llvm.memcpy/llvm.memmove or
-  /// llvm.memset call with the specified operands.
-  void EmitMemCpy(Value *DestPtr, Value *SrcPtr, Value *Size, unsigned Align);
-  void EmitMemMove(Value *DestPtr, Value *SrcPtr, Value *Size, unsigned Align);
-  void EmitMemSet(Value *DestPtr, Value *SrcVal, Value *Size, unsigned Align);
+  /// llvm.memset call with the specified operands.  Returns DestPtr bitcast
+  /// to i8*.
+  Value *EmitMemCpy(Value *DestPtr, Value *SrcPtr, Value *Size, unsigned Align);
+  Value *EmitMemMove(Value *DestPtr, Value *SrcPtr, Value *Size, unsigned Align);
+  Value *EmitMemSet(Value *DestPtr, Value *SrcVal, Value *Size, unsigned Align);
 
   /// EmitLandingPads - Emit EH landing pads.
   void EmitLandingPads();
@@ -429,8 +443,9 @@ private: // Helper functions.
   /// EmitUnwindBlock - Emit the lazily created EH unwind block.
   void EmitUnwindBlock();
 
-  /// EmitNoUnwindBlock - Emit the lazily created EH illegal-unwind block.
-  void EmitNoUnwindBlock();
+  /// EmitDebugInfo - Return true if debug info is to be emitted for current 
+  /// function.
+  bool EmitDebugInfo();
 
 private: // Helpers for exception handling.
 
@@ -443,16 +458,13 @@ private: // Helpers for exception handling.
   BasicBlock *getPostPad(unsigned RegionNo);
 
 private:
-  /// CreateTempLoc - Like CreateTemporary, but returns a MemRef.
-  MemRef CreateTempLoc(const Type *Ty);
-
   void EmitAutomaticVariableDecl(tree_node *decl);
 
   /// isNoopCast - Return true if a cast from V to Ty does not change any bits.
   ///
   static bool isNoopCast(Value *V, const Type *Ty);
 
-  void HandleMultiplyDefinedGCCTemp(tree_node *var);
+  void HandleMultiplyDefinedGimpleTemporary(tree_node *var);
   
   /// EmitAnnotateIntrinsic - Emits call to annotate attr intrinsic
   void EmitAnnotateIntrinsic(Value *V, tree_node *decl);
@@ -478,7 +490,7 @@ private:
   Value *EmitOBJ_TYPE_REF(tree_node *exp);
   Value *EmitCALL_EXPR(tree_node *exp, const MemRef *DestLoc);
   Value *EmitCallOf(Value *Callee, tree_node *exp, const MemRef *DestLoc,
-                    const ParamAttrsList *PAL);
+                    const AttrListPtr &PAL);
   Value *EmitMODIFY_EXPR(tree_node *exp, const MemRef *DestLoc);
   Value *EmitNOP_EXPR(tree_node *exp, const MemRef *DestLoc);
   Value *EmitCONVERT_EXPR(tree_node *exp, const MemRef *DestLoc);
@@ -490,7 +502,7 @@ private:
   Value *EmitTRUTH_NOT_EXPR(tree_node *exp);
   Value *EmitEXACT_DIV_EXPR(tree_node *exp, const MemRef *DestLoc);
   Value *EmitCompare(tree_node *exp, unsigned UIPred, unsigned SIPred, 
-                     unsigned FPOpc);
+                     unsigned FPPred, const Type *DestTy = 0);
   Value *EmitBinOp(tree_node *exp, const MemRef *DestLoc, unsigned Opc);
   Value *EmitPtrBinOp(tree_node *exp, unsigned Opc);
   Value *EmitTruthOp(tree_node *exp, unsigned Opc);
@@ -500,7 +512,9 @@ private:
                         unsigned Opc);
   Value *EmitFLOOR_MOD_EXPR(tree_node *exp, const MemRef *DestLoc);
   Value *EmitCEIL_DIV_EXPR(tree_node *exp);
+  Value *EmitFLOOR_DIV_EXPR(tree_node *exp);
   Value *EmitROUND_DIV_EXPR(tree_node *exp);
+  Value *EmitFieldAnnotation(Value *FieldPtr, tree_node *FieldDecl);
 
   // Exception Handling.
   Value *EmitEXC_PTR_EXPR(tree_node *exp);
@@ -513,20 +527,23 @@ private:
   void EmitModifyOfRegisterVariable(tree_node *vardecl, Value *RHS);
 
   // Helpers for Builtin Function Expansion.
+  void EmitMemoryBarrier(bool ll, bool ls, bool sl, bool ss, bool device);
   Value *BuildVector(const std::vector<Value*> &Elts);
   Value *BuildVector(Value *Elt, ...);
   Value *BuildVectorShuffle(Value *InVec1, Value *InVec2, ...);
+  Value *BuildBinaryAtomicBuiltin(tree_node *exp, Intrinsic::ID id);
+  Value *BuildCmpAndSwapAtomicBuiltin(tree_node *exp, tree_node *type, 
+                                      bool isBool);
 
   // Builtin Function Expansion.
   bool EmitBuiltinCall(tree_node *exp, tree_node *fndecl, 
                        const MemRef *DestLoc, Value *&Result);
   bool EmitFrontendExpandedBuiltinCall(tree_node *exp, tree_node *fndecl,
                                        const MemRef *DestLoc, Value *&Result);
-  bool EmitBuiltinUnaryIntOp(Value *InVal, Value *&Result, Intrinsic::ID Id);
-  Value *EmitBuiltinUnaryFPOp(Value *InVal, const char *F32Name,
-                              const char *F64Name, const char *LongDoubleName);
+  bool EmitBuiltinUnaryOp(Value *InVal, Value *&Result, Intrinsic::ID Id);
   Value *EmitBuiltinSQRT(tree_node *exp);
   Value *EmitBuiltinPOWI(tree_node *exp);
+  Value *EmitBuiltinPOW(tree_node *exp);
 
   bool EmitBuiltinConstantP(tree_node *exp, Value *&Result);
   bool EmitBuiltinAlloca(tree_node *exp, Value *&Result);
@@ -535,8 +552,9 @@ private:
   bool EmitBuiltinVAStart(tree_node *exp);
   bool EmitBuiltinVAEnd(tree_node *exp);
   bool EmitBuiltinVACopy(tree_node *exp);
-  bool EmitBuiltinMemCopy(tree_node *exp, Value *&Result, bool isMemMove);
-  bool EmitBuiltinMemSet(tree_node *exp, Value *&Result);
+  bool EmitBuiltinMemCopy(tree_node *exp, Value *&Result,
+                          bool isMemMove, bool SizeCheck);
+  bool EmitBuiltinMemSet(tree_node *exp, Value *&Result, bool SizeCheck);
   bool EmitBuiltinBZero(tree_node *exp, Value *&Result);
   bool EmitBuiltinPrefetch(tree_node *exp);
   bool EmitBuiltinReturnAddr(tree_node *exp, Value *&Result, bool isFrame);
@@ -560,14 +578,16 @@ private:
   Value *EmitComplexBinOp(tree_node *exp, const MemRef *DestLoc);
 
   // L-Value Expressions.
-  LValue EmitLV_DECL(tree_node *exp);
   LValue EmitLV_ARRAY_REF(tree_node *exp);
-  LValue EmitLV_COMPONENT_REF(tree_node *exp);
   LValue EmitLV_BIT_FIELD_REF(tree_node *exp);
-  LValue EmitLV_XXXXPART_EXPR(tree_node *exp, unsigned Idx);
-  LValue EmitLV_VIEW_CONVERT_EXPR(tree_node *exp);
+  LValue EmitLV_COMPONENT_REF(tree_node *exp);
+  LValue EmitLV_DECL(tree_node *exp);
   LValue EmitLV_EXC_PTR_EXPR(tree_node *exp);
   LValue EmitLV_FILTER_EXPR(tree_node *exp);
+  LValue EmitLV_INDIRECT_REF(tree_node *exp);
+  LValue EmitLV_VIEW_CONVERT_EXPR(tree_node *exp);
+  LValue EmitLV_WITH_SIZE_EXPR(tree_node *exp);
+  LValue EmitLV_XXXXPART_EXPR(tree_node *exp, unsigned Idx);
 
   // Constant Expressions.
   Value *EmitINTEGER_CST(tree_node *exp);
@@ -581,6 +601,10 @@ private:
                             Value *&Result,
                             const Type *ResultType,
                             std::vector<Value*> &Ops);
+
+public:
+  // Helper for taking the address of a label.
+  Constant *EmitLV_LABEL_DECL(tree_node *exp);
 };
 
 /// TreeConstantToLLVM - An instance of this class is created and used to 
@@ -615,5 +639,5 @@ public:
   
 };
 
-#endif
+#endif /* LLVM_INTERNAL_H */
 /* LLVM LOCAL end (ENTIRE FILE!)  */
