@@ -523,9 +523,14 @@ void set_flags_from_O (unsigned int cmdline)
           flag_tree_pre = 1;
 	}
       /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
       /* Enable loop unrolling at -O2 if -f[no-]unroll-loops is not used.  */
       if (!flag_unroll_loops_set && !optimize_size)
         flag_unroll_loops = 1;
+      /* Enable llvm inliner at -O2. */
+      if (cmdline)
+        flag_inline_functions = 1;
+#endif
       /* LLVM LOCAL end */
     }
 
@@ -604,7 +609,13 @@ decode_options (unsigned int argc, const char **argv)
     {
       if (!strcmp (argv[i], "-O"))
 	{
+          /* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
 	  optimize = 1;
+#else
+	  optimize = 2;
+#endif
+          /* LLVM LOCAL end */
 	  optimize_size = 0;
 	}
       else if (argv[i][0] == '-' && argv[i][1] == 'O')
@@ -779,8 +790,12 @@ decode_options (unsigned int argc, const char **argv)
       flag_strict_aliasing = saved_flag_strict_aliasing;
   /* APPLE LOCAL end AV 3846092 */
   /* APPLE LOCAL begin 4224227, 4231773 */
+  /* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
   if (!optimize_size_z)
     optimize_size = 0;
+#endif
+  /* LLVM LOCAL end */
   /* APPLE LOCAL end 4224227, 4231773 */
 }
 
@@ -897,9 +912,11 @@ common_handle_option (size_t scode, const char *arg, int value,
       break;
       
       /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
     case OPT_mllvm:
       llvm_optns = arg;
       break;
+#endif
       /* LLVM LOCAL end */
 
     case OPT_auxbase_strip:
@@ -1363,7 +1380,8 @@ print_target_help (void)
   for (i = 0; i < cl_options_count; i++)
     if ((cl_options[i].flags & (CL_TARGET | CL_UNDOCUMENTED)) == CL_TARGET)
       {
-	printf (_("\nTarget specific options:\n"));
+	/* APPLE LOCAL default to Wformat-security 5764921 */
+	printf ("%s", _("\nTarget specific options:\n"));
 	print_filtered_help (CL_TARGET);
 	break;
       }
