@@ -148,6 +148,15 @@ int reg_alloc_order[FIRST_PSEUDO_REGISTER] = REG_ALLOC_ORDER;
 int inv_reg_alloc_order[FIRST_PSEUDO_REGISTER];
 #endif
 
+/* APPLE LOCAL begin 5831562 add DIMODE_REG_ALLOC_ORDER */
+#ifdef DIMODE_REG_ALLOC_ORDER
+int dimode_reg_alloc_order[FIRST_PSEUDO_REGISTER] = DIMODE_REG_ALLOC_ORDER;
+
+/* The inverse of reg_alloc_order.  */
+int dimode_inv_reg_alloc_order[FIRST_PSEUDO_REGISTER];
+#endif
+/* APPLE LOCAL end 5831562 add DIMODE_REG_ALLOC_ORDER */
+
 /* For each reg class, a HARD_REG_SET saying which registers are in it.  */
 
 HARD_REG_SET reg_class_contents[N_REG_CLASSES];
@@ -167,6 +176,8 @@ static const unsigned int_reg_class_contents[N_REG_CLASSES][N_REG_INTS]
 
 unsigned int reg_class_size[N_REG_CLASSES];
 
+/* LLVM LOCAL */
+#ifndef ENABLE_LLVM
 /* For each reg class, table listing all the containing classes.  */
 
 static enum reg_class reg_class_superclasses[N_REG_CLASSES][N_REG_CLASSES];
@@ -174,6 +185,8 @@ static enum reg_class reg_class_superclasses[N_REG_CLASSES][N_REG_CLASSES];
 /* For each reg class, table listing all the classes contained in it.  */
 
 static enum reg_class reg_class_subclasses[N_REG_CLASSES][N_REG_CLASSES];
+/* LLVM LOCAL */
+#endif
 
 /* For each pair of reg classes,
    a largest reg class contained in their union.  */
@@ -298,6 +311,13 @@ init_reg_sets (void)
   for (i = 0; i < FIRST_PSEUDO_REGISTER; i++)
     inv_reg_alloc_order[reg_alloc_order[i]] = i;
 #endif
+
+/* APPLE LOCAL begin 5831562 add DIMODE_REG_ALLOC_ORDER */
+#ifdef DIMODE_REG_ALLOC_ORDER
+  for (i = 0; i < FIRST_PSEUDO_REGISTER; i++)
+    dimode_inv_reg_alloc_order[dimode_reg_alloc_order[i]] = i;
+#endif
+/* APPLE LOCAL end 5831562 add DIMODE_REG_ALLOC_ORDER */
 }
 
 /* After switches have been processed, which perhaps alter
@@ -312,9 +332,13 @@ init_reg_sets_1 (void)
   /* This macro allows the fixed or call-used registers
      and the register classes to depend on target flags.  */
 
+/* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
 #ifdef CONDITIONAL_REGISTER_USAGE
   CONDITIONAL_REGISTER_USAGE;
 #endif
+#endif
+/* LLVM LOCAL end */
 
   /* Compute number of hard regs in each class.  */
 
@@ -324,6 +348,8 @@ init_reg_sets_1 (void)
       if (TEST_HARD_REG_BIT (reg_class_contents[i], j))
 	reg_class_size[i]++;
 
+/* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
   /* Initialize the table of subunions.
      reg_class_subunion[I][J] gets the largest-numbered reg-class
      that is contained in the union of classes I and J.  */
@@ -471,6 +497,8 @@ init_reg_sets_1 (void)
       else if (CALL_REALLY_USED_REGNO_P (i))
 	SET_HARD_REG_BIT (regs_invalidated_by_call, i);
     }
+#endif
+/* LLVM LOCAL end */
 
   memset (have_regs_of_mode, 0, sizeof (have_regs_of_mode));
   memset (contains_reg_of_mode, 0, sizeof (contains_reg_of_mode));
@@ -486,6 +514,8 @@ init_reg_sets_1 (void)
 	       break;
 	     }
 
+/* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
   /* Initialize the move cost table.  Find every subset of each class
      and take the maximum cost of moving any subset to any other.  */
 
@@ -542,6 +572,8 @@ init_reg_sets_1 (void)
 		may_move_out_cost[m][i][j] = 65536;
 	      }
       }
+#endif
+/* LLVM LOCAL end */
 }
 
 /* Compute the table of register modes.
